import errno
import os
import subprocess

from io import StringIO
from tempfile import TemporaryDirectory
from unittest.mock import patch, MagicMock

from PIL import Image

from pyocr import builders
from pyocr import tesseract

from .tests_base import BaseTest


class TestTesseract(BaseTest):
    """
    These tests make sure the requirements for the tests are met.
    """
    def setUp(self):
        self.stdout = MagicMock()
        self.image = Image.new(mode="RGB", size=(1, 1))
        self.message = (
            b"tesseract 4.0.0\n leptonica-1.76.0\n"
            b"  libgif 5.1.4 : libjpeg 6b (libjpeg-turbo 1.5.2)"
            b" : libpng 1.6.34 "
            b": libtiff 4.0.9 : zlib 1.2.11 : libwebp 0.6.1"
            b" : libopenjp2 2.3.0\n"
            b" Found AVX2\n Found AVX\n Found SSE\n"
        )
        self.stdout.stdout.read.return_value = self.message
        self.stdout.wait.return_value = 0

    @patch("shutil.which")
    def test_available(self, which):
        which.return_value = True
        self.assertTrue(tesseract.is_available())
        which.assert_called_once_with("tesseract")

    @patch("subprocess.Popen")
    def test_version_error(self, popen):
        tesseract.g_version = None  # drop cached version
        self.stdout.wait.return_value = 2
        popen.return_value = self.stdout
        with self.assertRaises(tesseract.TesseractError) as te:
            tesseract.get_version()
        self.assertEqual(te.exception.status, 2)
        self.assertEqual(te.exception.message, self.message.decode())

    @patch("subprocess.Popen")
    def test_version_tesseract4(self, popen):
        tesseract.g_version = None  # drop cached version
        popen.return_value = self.stdout
        self.assertSequenceEqual(tesseract.get_version(), (4, 0, 0))

        # stderr must be explicitely ignored when calling 'tesseract -v'.
        # See https://gitlab.gnome.org/World/OpenPaperwork/pyocr/-/issues/118
        popen.assert_called_once()
        (args, kwargs) = popen.call_args
        self.assertNotIn('stderr', kwargs)

    @patch("subprocess.Popen")
    def test_version_tesseract4dev(self, popen):
        tesseract.g_version = None  # drop cached version
        message = self.message.replace(b"tesseract 4.0.0",
                                       b"tesseract 4.00.00dev2")
        self.stdout.stdout.read.return_value = message
        popen.return_value = self.stdout
        self.assertSequenceEqual(tesseract.get_version(), (4, 0, 0))

    @patch("subprocess.Popen")
    def test_version_tesseract4alpha(self, popen):
        tesseract.g_version = None  # drop cached version
        message = self.message.replace(b"tesseract 4.0.0",
                                       b"tesseract 4.00.00alpha")
        self.stdout.stdout.read.return_value = message
        popen.return_value = self.stdout
        self.assertSequenceEqual(tesseract.get_version(), (4, 0, 0))

    @patch("subprocess.Popen")
    def test_version_tesseract3(self, popen):
        tesseract.g_version = None  # drop cached version
        message = self.message.replace(b"tesseract 4.0.0",
                                       b"tesseract 3.05")
        self.stdout.stdout.read.return_value = message
        popen.return_value = self.stdout
        self.assertSequenceEqual(tesseract.get_version(), (3, 5, 0))

    @patch("subprocess.Popen")
    def test_version_tesseract3_no_minor(self, popen):
        tesseract.g_version = None  # drop cached version
        message = self.message.replace(b"tesseract 4.0.0",
                                       b"tesseract 3.0")
        self.stdout.stdout.read.return_value = message
        popen.return_value = self.stdout
        self.assertSequenceEqual(tesseract.get_version(), (3, 0, 0))

    @patch("subprocess.Popen")
    def test_version_windows(self, popen):
        tesseract.g_version = None  # drop cached version
        message = self.message.replace(b"tesseract 4.0.0",
                                       b"tesseract v4.0.0.20181030")
        self.stdout.stdout.read.return_value = message
        popen.return_value = self.stdout
        self.assertSequenceEqual(tesseract.get_version(), (4, 0, 0))

    @patch("subprocess.Popen")
    def test_version_cache(self, popen):
        """
        Make sure Tesseract is not called everytime we need the version.
        We need the version *often* in the code, and calling Tesseract
        everytime wouldn't be wise.
        """
        tesseract.g_version = None  # drop cached version

        self.stdout.stdout.read.return_value = self.message
        popen.return_value = self.stdout
        self.assertSequenceEqual(tesseract.get_version(), (4, 0, 0))

        self.stdout.stdout.read.return_value = "garbage"
        popen.return_value = self.stdout
        self.assertSequenceEqual(tesseract.get_version(), (4, 0, 0))

    @patch("subprocess.Popen")
    def test_version_error_splitting(self, popen):
        tesseract.g_version = None  # drop cached version
        message = self.message.replace(b"tesseract 4.0.0",
                                       b"tesseract 3")
        self.stdout.stdout.read.return_value = message
        popen.return_value = self.stdout
        with self.assertRaises(tesseract.TesseractError) as te:
            tesseract.get_version()
        self.assertEqual(te.exception.status, 0)
        self.assertIn("Unable to parse Tesseract version (spliting failed): ",
                      te.exception.message)

    @patch("subprocess.Popen")
    def test_version_error_nan(self, popen):
        tesseract.g_version = None  # drop cached version
        message = self.message.replace(b"tesseract 4.0.0",
                                       b"tesseract A.B.C")
        self.stdout.stdout.read.return_value = message
        popen.return_value = self.stdout
        with self.assertRaises(tesseract.TesseractError) as te:
            tesseract.get_version()
        self.assertEqual(te.exception.status, 0)
        self.assertIn("Unable to parse Tesseract version (not a number): ",
                      te.exception.message)

    @patch("subprocess.Popen")
    def test_langs(self, popen):
        self.stdout.stdout.read.return_value = (
            b"List of available languages (4):\n"
            b"eng\n"
            b"fra\n"
            b"jpn\n"
            b"osd\n"
        )
        popen.return_value = self.stdout
        langs = tesseract.get_available_languages()
        for lang in ("eng", "fra", "jpn", "osd"):
            self.assertIn(lang, langs)
        popen.assert_called_once_with(
            ["tesseract", "--list-langs"],
            startupinfo=None, creationflags=0,
            stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )

    @patch("subprocess.Popen")
    def test_langs_error(self, popen):
        self.stdout.stdout.read.return_value = b"No languages\n"
        self.stdout.wait.return_value = 1
        popen.return_value = self.stdout
        with self.assertRaises(tesseract.TesseractError) as te:
            tesseract.get_available_languages()
        self.assertEqual(te.exception.status, 1)
        self.assertEqual("unable to get languages", te.exception.message)
        popen.assert_called_once_with(
            ["tesseract", "--list-langs"],
            startupinfo=None, creationflags=0,
            stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )

    @patch("pyocr.tesseract.get_available_languages")
    @patch("pyocr.tesseract.get_version")
    def test_can_detect_orientation_tesseract4(self, get_version,
                                               get_available_languages):
        get_version.return_value = (4, 0, 0)

        get_available_languages.return_value = ['eng', 'fra', 'jpn', 'osd']
        self.assertTrue(tesseract.can_detect_orientation())

        get_available_languages.return_value = ['eng', 'fra', 'jpn']
        self.assertFalse(tesseract.can_detect_orientation())

    @patch("pyocr.tesseract.get_available_languages")
    @patch("pyocr.tesseract.get_version")
    def test_can_detect_orientation_tesseract3(
                self, get_version, get_available_languages
            ):
        get_available_languages.return_value = ['eng', 'fra', 'jpn', 'osd']
        get_version.return_value = (3, 3, 0)
        self.assertTrue(tesseract.can_detect_orientation())

    @patch("pyocr.tesseract.get_available_languages")
    @patch("pyocr.tesseract.get_version")
    def test_cannot_detect_orientation_tesseract3(
                self, get_version, get_available_languages
            ):
        get_available_languages.return_value = ['eng', 'fra', 'jpn', 'osd']
        get_version.return_value = (3, 2, 1)
        self.assertFalse(tesseract.can_detect_orientation())

    def test_name(self):
        self.assertEqual(tesseract.get_name(), "Tesseract (sh)")

    @patch("pyocr.tesseract.get_version")
    def test_psm_parameter(self, get_version):
        get_version.return_value = (3, 5, 0)
        self.assertEqual(tesseract.psm_parameter(), "-psm")
        get_version.return_value = (4, 0, 0)
        self.assertEqual(tesseract.psm_parameter(), "--psm")

    def test_available_builders(self):
        self.assertListEqual(
            tesseract.get_available_builders(),
            [
                builders.LineBoxBuilder,
                builders.TextBuilder,
                builders.WordBoxBuilder,
                tesseract.CharBoxBuilder,
                builders.DigitBuilder,
                builders.DigitLineBoxBuilder,
            ]
        )

    @patch("pyocr.tesseract.get_version")
    @patch("subprocess.Popen")
    def test_run_tesseract(self, popen, get_version):
        message = (
            b"Tesseract Open Source OCR Engine v4.0.0 with Leptonica\n"
        )
        self.stdout.stdout.read.return_value = message
        popen.return_value = self.stdout

        with TemporaryDirectory() as tmpdir:
            self.image.save(os.path.join(tmpdir, "input.bmp"))
            status, error = tesseract.run_tesseract(
                "input.bmp",
                "output",
                cwd=tmpdir,
            )
        self.assertEqual(status, 0)
        self.assertEqual(error, message)
        popen.assert_called_once_with(
            ["tesseract", "input.bmp", "output"],
            cwd=tmpdir,
            startupinfo=None,
            creationflags=0,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT
        )

        get_version.return_value = (4, 0, 0)
        builder = builders.TextBuilder()
        with TemporaryDirectory() as tmpdir:
            self.image.save(os.path.join(tmpdir, "input2.bmp"))
            status, error = tesseract.run_tesseract(
                "input2.bmp",
                "output2",
                cwd=tmpdir,
                lang="fra",
                flags=builder.tesseract_flags,
                configs=builder.tesseract_configs,
            )
        self.assertEqual(status, 0)
        self.assertEqual(error, message)
        popen.assert_called_with(
            ["tesseract", "input2.bmp", "output2", "-l", "fra", "--psm", "3"],
            cwd=tmpdir,
            startupinfo=None,
            creationflags=0,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT
        )
        self.assertEqual(popen.call_count, 2)

    @patch("pyocr.tesseract.get_version")
    @patch("tempfile.TemporaryDirectory")
    @patch("subprocess.Popen")
    def test_detect_orientation_tesseract4(self, popen, temp_dir, get_version):
        get_version.return_value = (4, 0, 0)
        self.stdout.stdout.read.return_value = (
            b"Page number: 0\n"
            b"Orientation in degrees: 90\n"
            b"Rotate: 270\n"
            b"Orientation confidence: 9.30\n"
            b"Script: Latin\n"
            b"Script confidence: 8.06\n"
        )
        popen.return_value = self.stdout
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            result = tesseract.detect_orientation(self.image)
            self.assertEqual(result["angle"], 90)
            self.assertEqual(result["confidence"], 9.30)
            popen.assert_called_once_with(
                ["tesseract", "input.bmp", "stdout", "--psm", "0"],
                stdin=subprocess.PIPE,
                shell=False,
                startupinfo=None,
                creationflags=0,
                cwd=tmpdir,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT
            )

    @patch("pyocr.tesseract.get_version")
    @patch("tempfile.TemporaryDirectory")
    @patch("subprocess.Popen")
    def test_detect_orientation_tesseract4_non_rgb_image(self, popen, temp_dir,
                                                         get_version):
        """This tests that detect_orientation works with non RGB mode images and
        that image is converted in function."""
        image = self.image.convert("L")
        get_version.return_value = (4, 0, 0)
        self.stdout.stdout.read.return_value = (
            b"Page number: 0\n"
            b"Orientation in degrees: 90\n"
            b"Rotate: 270\n"
            b"Orientation confidence: 9.30\n"
            b"Script: Latin\n"
            b"Script confidence: 8.06\n"
        )
        popen.return_value = self.stdout
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            result = tesseract.detect_orientation(image)
            self.assertEqual(result["angle"], 90)
            self.assertEqual(result["confidence"], 9.30)
            popen.assert_called_once_with(
                ["tesseract", "input.bmp", "stdout", "--psm", "0"],
                stdin=subprocess.PIPE,
                shell=False,
                startupinfo=None,
                creationflags=0,
                cwd=tmpdir,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT
            )

    @patch("pyocr.tesseract.get_version")
    @patch("tempfile.TemporaryDirectory")
    @patch("subprocess.Popen")
    def test_detect_orientation_tesseract4_with_lang(self, popen, temp_dir,
                                                     get_version):
        get_version.return_value = (4, 0, 0)
        self.stdout.stdout.read.return_value = (
            b"Page number: 0\n"
            b"Orientation in degrees: 90\n"
            b"Rotate: 270\n"
            b"Orientation confidence: 9.30\n"
            b"Script: Latin\n"
            b"Script confidence: 8.06\n"
        )
        popen.return_value = self.stdout
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            result = tesseract.detect_orientation(self.image, lang="fra")
            self.assertEqual(result["angle"], 90)
            self.assertEqual(result["confidence"], 9.30)
            popen.assert_called_once_with(
                ["tesseract", "input.bmp", "stdout",
                 "--psm", "0", "-l", "osd"],
                stdin=subprocess.PIPE,
                shell=False,
                startupinfo=None,
                creationflags=0,
                cwd=tmpdir,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT
            )

    @patch("pyocr.tesseract.get_version")
    @patch("tempfile.TemporaryDirectory")
    @patch("subprocess.Popen")
    def test_detect_orientation_tesseract4_error(self, popen, temp_dir,
                                                 get_version):
        get_version.return_value = (4, 0, 0)
        self.stdout.stdout.read.return_value = (
            b"Could not initialize tesseract\n"
        )
        popen.return_value = self.stdout
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with self.assertRaises(tesseract.TesseractError) as te:
                tesseract.detect_orientation(self.image)
            popen.assert_called_once_with(
                ["tesseract", "input.bmp", "stdout", "--psm", "0"],
                stdin=subprocess.PIPE,
                shell=False,
                startupinfo=None,
                creationflags=0,
                cwd=tmpdir,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT
            )
            self.assertEqual(te.exception.status, -1)
            self.assertIn("Error initializing tesseract", te.exception.message)

    @patch("pyocr.tesseract.get_version")
    @patch("tempfile.TemporaryDirectory")
    @patch("subprocess.Popen")
    def test_detect_orientation_tesseract4_bad_output(self, popen, temp_dir,
                                                      get_version):
        get_version.return_value = (4, 0, 0)
        self.stdout.stdout.read.return_value = (
            b"Page number: 0\n"
            b"Orientation in degrees: ABC\n"
            b"Rotate: 270\n"
            b"Orientation confidence: AB.CD\n"
            b"Script: Latin\n"
            b"Script confidence: 8.06\n"
        )
        popen.return_value = self.stdout
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with self.assertRaises(tesseract.TesseractError) as te:
                tesseract.detect_orientation(self.image)
            popen.assert_called_once_with(
                ["tesseract", "input.bmp", "stdout", "--psm", "0"],
                stdin=subprocess.PIPE,
                shell=False,
                startupinfo=None,
                creationflags=0,
                cwd=tmpdir,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT
            )
            self.assertEqual(te.exception.status, -1)
            self.assertIn("No script found in image", te.exception.message)

    @patch("pyocr.tesseract.get_version")
    @patch("tempfile.TemporaryDirectory")
    @patch("subprocess.Popen")
    def test_detect_orientation_tesseract3(self, popen, temp_dir, get_version):
        get_version.return_value = (3, 5, 0)
        self.stdout.stdout.read.return_value = (
            b"Page number: 0\n"
            b"Orientation in degrees: 90\n"
            b"Rotate: 270\n"
            b"Orientation confidence: 9.30\n"
            b"Script: Latin\n"
            b"Script confidence: 8.06\n"
        )
        popen.return_value = self.stdout
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            result = tesseract.detect_orientation(self.image)
            self.assertEqual(result["angle"], 90)
            self.assertEqual(result["confidence"], 9.30)
            popen.assert_called_once_with(
                ["tesseract", "input.bmp", "stdout", "-psm", "0"],
                stdin=subprocess.PIPE,
                shell=False,
                startupinfo=None,
                creationflags=0,
                cwd=tmpdir,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT
            )

    @patch("pyocr.tesseract.get_version")
    @patch("tempfile.TemporaryDirectory")
    @patch("subprocess.Popen")
    def test_detect_orientation_tesseract3_with_lang(self, popen, temp_dir,
                                                     get_version):
        get_version.return_value = (3, 5, 0)
        self.stdout.stdout.read.return_value = (
            b"Page number: 0\n"
            b"Orientation in degrees: 90\n"
            b"Rotate: 270\n"
            b"Orientation confidence: 9.30\n"
            b"Script: Latin\n"
            b"Script confidence: 8.06\n"
        )
        popen.return_value = self.stdout
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            result = tesseract.detect_orientation(self.image, lang="fra")
            self.assertEqual(result["angle"], 90)
            self.assertEqual(result["confidence"], 9.30)
            popen.assert_called_once_with(
                ["tesseract", "input.bmp", "stdout", "-psm", "0", "-l", "fra"],
                stdin=subprocess.PIPE,
                shell=False,
                startupinfo=None,
                creationflags=0,
                cwd=tmpdir,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT
            )

    @patch("pyocr.tesseract.get_version")
    @patch("tempfile.TemporaryDirectory")
    @patch("subprocess.Popen")
    def test_detect_orientation_tesseract3_error(self, popen, temp_dir,
                                                 get_version):
        get_version.return_value = (3, 5, 0)
        self.stdout.stdout.read.return_value = (
            b"Could not initialize tesseract\n"
        )
        popen.return_value = self.stdout
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with self.assertRaises(tesseract.TesseractError) as te:
                tesseract.detect_orientation(self.image)
            popen.assert_called_once_with(
                ["tesseract", "input.bmp", "stdout", "-psm", "0"],
                stdin=subprocess.PIPE,
                shell=False,
                startupinfo=None,
                creationflags=0,
                cwd=tmpdir,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT
            )
            self.assertEqual(te.exception.status, -1)
            self.assertIn("Error initializing tesseract", te.exception.message)

    @patch("pyocr.tesseract.get_version")
    @patch("tempfile.TemporaryDirectory")
    @patch("subprocess.Popen")
    def test_detect_orientation_tesseract3_bad_output(self, popen, temp_dir,
                                                      get_version):
        get_version.return_value = (3, 5, 0)
        self.stdout.stdout.read.return_value = (
            b"Page number: 0\n"
            b"Orientation in degrees: ABC\n"
            b"Rotate: 270\n"
            b"Orientation confidence: AB.CD\n"
            b"Script: Latin\n"
            b"Script confidence: 8.06\n"
        )
        popen.return_value = self.stdout
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with self.assertRaises(tesseract.TesseractError) as te:
                tesseract.detect_orientation(self.image)
            popen.assert_called_once_with(
                ["tesseract", "input.bmp", "stdout", "-psm", "0"],
                stdin=subprocess.PIPE,
                shell=False,
                startupinfo=None,
                creationflags=0,
                cwd=tmpdir,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT
            )
            self.assertEqual(te.exception.status, -1)
            self.assertIn("No script found in image", te.exception.message)


class TestTesseractTxt(BaseTest):
    """
    These tests make sure the "usual" OCR works fine. (the one generating
    a .txt file)
    """
    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.image = Image.new(mode="RGB", size=(1, 1))
        self.builder = builders.TextBuilder()

    @patch("pyocr.tesseract.get_version")
    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_image_to_string_defaults_to_text_buidler(self, run_tesseract,
                                                      copen, temp_dir,
                                                      get_version):
        get_version.return_value = (4, 0, 0)
        run_tesseract.return_value = (0, "")
        copen.return_value = StringIO(self._get_file_content("text"))
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            result = tesseract.image_to_string(self.image)

        self.assertEqual(result, self._get_file_content("text").strip())
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_lang(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (0, "")
        copen.return_value = StringIO(self._get_file_content("text"))
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            result = tesseract.image_to_string(self.image, lang="fra",
                                               builder=self.builder)

        self.assertEqual(result, self._get_file_content("text").strip())
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang="fra",
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_text(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (0, "")
        copen.return_value = StringIO(self._get_file_content("text"))
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            result = tesseract.image_to_string(self.image,
                                               builder=self.builder)

        self.assertEqual(result, self._get_file_content("text").strip())
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_text_non_rgb_image(self, run_tesseract, copen, temp_dir):
        """This tests that image_to_string works with non RGB mode images and
        that image is converted in function."""
        image = self.image.convert("L")
        run_tesseract.return_value = (0, "")
        copen.return_value = StringIO(self._get_file_content("text"))
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            result = tesseract.image_to_string(image, builder=self.builder)

        self.assertEqual(result, self._get_file_content("text").strip())
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_text_error(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (1, "Error")
        copen.return_value = StringIO(self._get_file_content("text"))
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with self.assertRaises(tesseract.TesseractError) as te:
                tesseract.image_to_string(self.image, builder=self.builder)
        self.assertEqual(te.exception.status, 1)
        self.assertEqual(te.exception.message, "Error")
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_text_error_file(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (0, "")
        copen.side_effect = Exception("Unknown error")
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with self.assertRaises(Exception):
                tesseract.image_to_string(self.image, builder=self.builder)
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_text_cannot_open_file(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (0, "")
        copen.side_effect = PermissionError(errno.EPERM, "Error opening file")
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with self.assertRaises(PermissionError):
                tesseract.image_to_string(self.image, builder=self.builder)
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_text_no_output(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (0, "No file output")
        copen.side_effect = FileNotFoundError(
            errno.ENOENT,
            "[Errno 2] No such file or directory: 'output'"
        )
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with self.assertRaises(tesseract.TesseractError) as te:
                tesseract.image_to_string(self.image, builder=self.builder)
        self.assertEqual(te.exception.status, -1)
        self.assertIn("Unable to find output file (tested",
                      te.exception.message)
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )


class TestTesseractCharBox(BaseTest):
    """
    These tests make sure that Tesseract box handling works fine.
    """
    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.image = Image.new(mode="RGB", size=(1, 1))
        self.builder = tesseract.CharBoxBuilder()

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_char(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (0, "")
        copen.return_value = StringIO(self._get_file_content("boxes"))
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            result = tesseract.image_to_string(self.image,
                                               builder=self.builder)

        for box in result:
            self.assertIsInstance(box, builders.Box)
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_char_error(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (1, "Error")
        copen.return_value = StringIO(self._get_file_content("boxes"))
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with self.assertRaises(tesseract.TesseractError) as te:
                tesseract.image_to_string(self.image, builder=self.builder)
        self.assertEqual(te.exception.status, 1)
        self.assertEqual(te.exception.message, "Error")
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_char_no_output(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (0, "No file output")
        copen.side_effect = FileNotFoundError(
            errno.ENOENT,
            "[Errno 2] No such file or directory: 'output'"
        )
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with self.assertRaises(tesseract.TesseractError) as te:
                tesseract.image_to_string(self.image, builder=self.builder)
        self.assertEqual(te.exception.status, -1)
        self.assertIn("Unable to find output file (tested",
                      te.exception.message)
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )


class TestCharBoxBuilder(BaseTest):
    def test_init(self):
        builder = tesseract.CharBoxBuilder()
        self.assertListEqual(builder.file_extensions, ["box"])
        self.assertListEqual(builder.tesseract_flags, [])
        self.assertListEqual(
            builder.tesseract_configs,
            ["batch.nochop", "makebox"]
        )
        self.assertListEqual(builder.cuneiform_args, [])
        self.assertEqual(builder.tesseract_layout, 1)

    def test_read_file(self):
        builder = tesseract.CharBoxBuilder()
        boxes = builder.read_file(self._get_file_handle("boxes"))
        for box in boxes:
            self.assertIsInstance(box, builders.Box)

    def test_read_empty_file(self):
        builder = tesseract.CharBoxBuilder()
        output = StringIO()
        self.assertListEqual(builder.read_file(output), [])

    def test_read_file_empty_lines(self):
        builder = tesseract.CharBoxBuilder()
        boxes = builder.read_file(self._get_file_handle("boxes_empty_lines"))
        for box in boxes:
            self.assertIsInstance(box, builders.Box)
            self.assertNotEqual(box.content, "")

    def test_read_file_short_lines(self):
        builder = tesseract.CharBoxBuilder()
        boxes = builder.read_file(self._get_file_handle("boxes_short_lines"))
        for box in boxes:
            self.assertIsInstance(box, builders.Box)
            self.assertNotEqual(box.content, "#")

    def test_write_file(self):
        builder = tesseract.CharBoxBuilder()
        output = StringIO()
        boxes = [
            builders.Box("a", ((10, 11), (12, 13)), 95),
            builders.Box("b", ((11, 12), (13, 14))),
            builders.Box("c", ((12, 13), (14, 15))),
            builders.Box("d", ((13, 14), (15, 16)), 87),
            builders.Box("\xe9", ((14, 15), (16, 17)), 88),
        ]
        builder.write_file(output, boxes)
        output.seek(0)
        output = output.read()
        for box in boxes:
            self.assertIn(box.content, output)
            self.assertIn("{} {} {} {}".format(
                box.position[0][0], box.position[0][1],
                box.position[1][0], box.position[1][1],
            ), output)

    def test_str_method(self):
        self.assertEqual(str(tesseract.CharBoxBuilder()), "Character boxes")


class TestTesseractDigits(BaseTest):

    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.builder = builders.DigitBuilder()
        self.image = Image.new(mode="RGB", size=(1, 1))

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_digits(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (0, "")
        copen.return_value = StringIO(self._get_file_content("digits"))
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with open(os.path.join(tmpdir, "output.txt"), "w") as fh:
                fh.write("")
            result = tesseract.image_to_string(self.image,
                                               builder=self.builder)

        for digit in result:
            self.assertIsInstance(int(digit), int)
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )


class TestTesseractWordBox(BaseTest):

    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.image = Image.new(mode="RGB", size=(1, 1))
        self.builder = builders.WordBoxBuilder()

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_word(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (0, "")
        copen.return_value = StringIO(self._get_file_content("words"))
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with open(os.path.join(tmpdir, "output.hocr"), "w") as fh:
                fh.write("")
            result = tesseract.image_to_string(self.image,
                                               builder=self.builder)

        for box in result:
            self.assertIsInstance(box, builders.Box)
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_word_error(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (1, "Error")
        copen.return_value = StringIO(self._get_file_content("words"))
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with self.assertRaises(tesseract.TesseractError) as te:
                tesseract.image_to_string(self.image, builder=self.builder)

        self.assertEqual(te.exception.status, 1)
        self.assertEqual(te.exception.message, "Error")
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_word_no_output(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (0, "No file output")
        copen.side_effect = FileNotFoundError(
            errno.ENOENT,
            "[Errno 2] No such file or directory: 'output'"
        )
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with self.assertRaises(tesseract.TesseractError) as te:
                tesseract.image_to_string(self.image, builder=self.builder)

        self.assertEqual(te.exception.status, -1)
        self.assertIn("Unable to find output file (tested",
                      te.exception.message)
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )


class TestTesseractLineBox(BaseTest):

    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.image = Image.new(mode="RGB", size=(1, 1))
        self.builder = builders.LineBoxBuilder()

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_line(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (0, "")
        copen.return_value = StringIO(
            self._get_file_content("tesseract.lines")
        )
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with open(os.path.join(tmpdir, "output.hocr"), "w") as fh:
                fh.write("")
            result = tesseract.image_to_string(self.image,
                                               builder=self.builder)

        for line in result:
            self.assertIsInstance(line, builders.LineBox)
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_line_error(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (1, "Error")
        copen.return_value = StringIO(
            self._get_file_content("tesseract.lines")
        )
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with self.assertRaises(tesseract.TesseractError) as te:
                tesseract.image_to_string(self.image, builder=self.builder)

        self.assertEqual(te.exception.status, 1)
        self.assertEqual(te.exception.message, "Error")
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_line_no_output(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (0, "No file output")
        copen.side_effect = FileNotFoundError(
            errno.ENOENT,
            "[Errno 2] No such file or directory: 'output'"
        )
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with self.assertRaises(tesseract.TesseractError) as te:
                tesseract.image_to_string(self.image, builder=self.builder)

        self.assertEqual(te.exception.status, -1)
        self.assertIn("Unable to find output file (tested",
                      te.exception.message)
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )


class TestTesseractDigitsLineBox(BaseTest):

    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.image = Image.new(mode="RGB", size=(1, 1))
        self.builder = builders.DigitLineBoxBuilder()

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_line(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (0, "")
        copen.return_value = StringIO(self._get_file_content("digits.lines"))
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with open(os.path.join(tmpdir, "output.hocr"), "w") as fh:
                fh.write("")
            result = tesseract.image_to_string(self.image,
                                               builder=self.builder)

        for line in result:
            self.assertIsInstance(line, builders.LineBox)
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_line_error(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (1, "Error")
        copen.return_value = StringIO(self._get_file_content("digits.lines"))
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with self.assertRaises(tesseract.TesseractError) as te:
                tesseract.image_to_string(self.image, builder=self.builder)

        self.assertEqual(te.exception.status, 1)
        self.assertEqual(te.exception.message, "Error")
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )

    @patch("tempfile.TemporaryDirectory")
    @patch("codecs.open")
    @patch("pyocr.tesseract.run_tesseract")
    def test_line_no_output(self, run_tesseract, copen, temp_dir):
        run_tesseract.return_value = (0, "No file output")
        copen.side_effect = FileNotFoundError(
            errno.ENOENT,
            "[Errno 2] No such file or directory: 'output'"
        )
        with TemporaryDirectory(prefix="tess_") as tmpdir:
            enter = MagicMock()
            enter.__enter__.return_value = tmpdir
            temp_dir.return_value = enter
            with self.assertRaises(tesseract.TesseractError) as te:
                tesseract.image_to_string(self.image, builder=self.builder)

        self.assertEqual(te.exception.status, -1)
        self.assertIn("Unable to find output file (tested",
                      te.exception.message)
        run_tesseract.assert_called_once_with(
            "input.bmp", "output", cwd=tmpdir, lang=None,
            flags=self.builder.tesseract_flags,
            configs=self.builder.tesseract_configs,
        )



import unittest

from io import StringIO
from itertools import product
from random import randint
from unittest.mock import patch

from pyocr import builders

from .tests_base import BaseTest


class TestTextBuilder(unittest.TestCase):
    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.builder = builders.TextBuilder()

    def test_init(self):
        self.assertListEqual(self.builder.file_extensions, ["txt"])
        self.assertListEqual(self.builder.tesseract_flags, ["--psm", "3"])
        self.assertListEqual(self.builder.tesseract_configs, [])
        self.assertEqual(self.builder.tesseract_layout, 3)
        self.assertListEqual(self.builder.cuneiform_args, ["-f", "text"])

    @patch("pyocr.tesseract.get_version")
    def test_init_cuneiform_params(self, get_version):
        get_version.return_value = (4, 0, 0)
        # XXX Maybe overkill?
        # this check all combinations of parameters
        for cun_dotmat, cun_fax, cun_sglcol in product(*((False, True),) * 3):
            builder = builders.TextBuilder(
                cuneiform_dotmatrix=cun_dotmat,
                cuneiform_fax=cun_fax,
                cuneiform_singlecolumn=cun_sglcol
            )
            if cun_dotmat:
                self.assertIn("--dotmatrix", builder.cuneiform_args)
            else:
                self.assertNotIn("--dotmatrix", builder.cuneiform_args)
            if cun_fax:
                self.assertIn("--fax", builder.cuneiform_args)
            else:
                self.assertNotIn("--fax", builder.cuneiform_args)
            if cun_sglcol:
                self.assertIn("--singlecolumn", builder.cuneiform_args)
            else:
                self.assertNotIn("--singlecolumn", builder.cuneiform_args)

    def test_read_file(self):
        txt = "first line\nsecond line\n0123456789\n🖨  "
        input_fh = StringIO(txt)
        output = self.builder.read_file(input_fh)
        self.assertEqual(output, txt.strip())

    def test_write_file(self):
        output = StringIO()
        txt = "first line\nsecond line\n0123456789\n🖨  "
        self.builder.write_file(output, txt)
        output.seek(0)
        self.assertEqual(output.read(), txt)

    def test_start_line(self):
        box = builders.Box("word", ((10, 11), (12, 13)))
        self.builder.start_line(box)
        self.assertListEqual(self.builder.built_text, [""])

    def test_add_word_no_line(self):
        box = builders.Box("word", ((10, 11), (12, 13)))
        with self.assertRaises(IndexError):
            self.builder.add_word(box.content, box)

    def test_add_word(self):
        box = builders.Box("word", ((10, 11), (12, 13)))
        self.builder.start_line(box)
        self.builder.add_word(box.content, box)
        self.assertEqual(self.builder.built_text[0], box.content)
        self.builder.add_word(box.content, box)
        self.assertEqual(self.builder.built_text[0],
                         box.content + " " + box.content)

    def test_end_line(self):
        before = list(self.builder.built_text)
        self.builder.end_line()
        self.assertEqual(self.builder.built_text, before)

    def test_get_output(self):
        box = builders.Box("word", ((10, 11), (12, 13)))
        self.builder.start_line(box)
        self.builder.add_word("word1", box)
        self.builder.add_word("word2", box)
        self.builder.start_line(box)
        self.builder.add_word("word3", box)
        self.builder.add_word("word4", box)
        self.assertEqual(self.builder.get_output(), "word1 word2\nword3 word4")

    def test_str_method(self):
        self.assertEqual(str(self.builder), "Raw text")


class TestWordBoxBuilder(BaseTest):

    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.builder = builders.WordBoxBuilder()

    @patch("pyocr.tesseract.get_version")
    def test_init_tesseract_version_3(self, get_version):
        for version in range(6):
            get_version.return_value = (3, version, 0)
            builder = builders.WordBoxBuilder()
            self.assertListEqual(builder.tesseract_flags, ["-psm", "1"])
            self.assertListEqual(builder.file_extensions, ["html", "hocr"])
            self.assertListEqual(builder.tesseract_configs, ["hocr"])
            self.assertListEqual(builder.cuneiform_args, ["-f", "hocr"])
            self.assertListEqual(builder.word_boxes, [])
            self.assertEqual(builder.tesseract_layout, 1)

    @patch("pyocr.tesseract.get_version")
    def test_init_tesseract_version_4(self, get_version):
        get_version.return_value = (4, 0, 0)
        builder = builders.WordBoxBuilder()
        self.assertListEqual(builder.tesseract_flags, ["--psm", "1"])
        self.assertListEqual(builder.file_extensions, ["html", "hocr"])
        self.assertListEqual(builder.tesseract_configs, ["hocr"])
        self.assertListEqual(builder.cuneiform_args, ["-f", "hocr"])
        self.assertListEqual(builder.word_boxes, [])
        self.assertEqual(builder.tesseract_layout, 1)

    def test_read_file(self):
        words = self.builder.read_file(self._get_file_handle("words"))
        for word in words:
            self.assertIsInstance(word, builders.Box)

    def test_empty_read_file(self):
        output = StringIO()
        self.assertListEqual(self.builder.read_file(output), [])

    def test_read_file_bbox(self):
        words = self.builder.read_file(self._get_file_handle("words_bbox"))
        for word in words:
            self.assertIsInstance(word, builders.Box)
        self.assertNotEqual(words[-1].content, "preguieoso.")

    def test_write_file(self):
        output = StringIO()
        boxes = [
            builders.Box("word1", ((10, 11), (12, 13)), 95),
            builders.Box("word2", ((11, 12), (13, 14))),
            builders.Box("word3", ((12, 13), (14, 15))),
            builders.Box("word4", ((13, 14), (15, 16)), 87),
        ]
        self.builder.write_file(output, boxes)
        output.seek(0)
        output = output.read()
        for box in boxes:
            self.assertIn(box.content, output)
            self.assertIn("{} {} {} {}".format(
                box.position[0][0], box.position[0][1],
                box.position[1][0], box.position[1][1],
            ), output)
            self.assertIn(str(box.confidence), output)

    def test_start_line(self):
        box = builders.Box("word", ((1, 2), (3, 4)))
        before = list(self.builder.word_boxes)
        self.builder.start_line(box)
        self.assertEqual(self.builder.word_boxes, before)

    def test_add_word(self):
        box = builders.Box("word", ((1, 2), (3, 4)), 42)
        self.builder.add_word(box.content, box.position, box.confidence)
        for box in self.builder.word_boxes:
            self.assertIsInstance(box, builders.Box)
        self.assertEqual(self.builder.word_boxes[0], box)

    def test_end_line(self):
        before = list(self.builder.word_boxes)
        self.builder.end_line()
        self.assertEqual(self.builder.word_boxes, before)

    def test_get_output(self):
        boxes = [
            builders.Box("word1", ((10, 11), (12, 13)), 95),
            builders.Box("word2", ((11, 12), (13, 14))),
            builders.Box("word3", ((12, 13), (14, 15))),
            builders.Box("word4", ((13, 14), (15, 16)), 87),
        ]
        for box in boxes:
            self.builder.add_word(box.content, box.position, box.confidence)
        output = self.builder.get_output()
        for box, box_expected in zip(output, boxes):
            self.assertIsInstance(box, builders.Box)
            self.assertEqual(box, box_expected)
            self.assertEqual(box.content, box_expected.content)

    def test_str_method(self):
        self.assertEqual(str(self.builder), "Word boxes")


class TestLineBoxBuilder(BaseTest):

    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.builder = builders.LineBoxBuilder()

    @patch("pyocr.tesseract.get_version")
    def test_init_tesseract_version_3(self, get_version):
        for version in range(6):
            get_version.return_value = (3, version, 0)
            builder = builders.LineBoxBuilder()
            self.assertListEqual(builder.tesseract_flags, ["-psm", "1"])
            self.assertListEqual(builder.file_extensions, ["html", "hocr"])
            self.assertListEqual(builder.tesseract_configs, ["hocr"])
            self.assertListEqual(builder.cuneiform_args, ["-f", "hocr"])
            self.assertListEqual(builder.lines, [])
            self.assertEqual(builder.tesseract_layout, 1)

    @patch("pyocr.tesseract.get_version")
    def test_init_tesseract_version_4(self, get_version):
        get_version.return_value = (4, 0, 0)
        builder = builders.LineBoxBuilder()
        self.assertListEqual(builder.tesseract_flags, ["--psm", "1"])
        self.assertListEqual(builder.file_extensions, ["html", "hocr"])
        self.assertListEqual(builder.tesseract_configs, ["hocr"])
        self.assertListEqual(builder.cuneiform_args, ["-f", "hocr"])
        self.assertListEqual(builder.lines, [])
        self.assertEqual(builder.tesseract_layout, 1)

    def test_read_file(self):
        for input_fh in (self._get_file_handle("tesseract.lines"),
                         self._get_file_handle("cuneiform.lines")):
            lines = self.builder.read_file(input_fh)
            for line in lines:
                self.assertIsInstance(line, builders.LineBox)

    def test_empty_read_file(self):
        empty = StringIO()
        self.assertListEqual(self.builder.read_file(empty), [])

    def test_write_file(self):
        output_fh = StringIO()
        lines = []
        for l in range(4):
            boxes = []
            for b in range(4):
                word = "word" + str(4*l+b)
                position = ((4*l+b, 4*l+b+1), (4*l+b+2, 4*l+b+3))
                boxes.append(builders.Box(word, position, randint(0, 100)))
            line_position = ((4*l, 4*(l+1)), (4*l+2, 4*(l+1)+2))
            lines.append(builders.LineBox(boxes, line_position))
        self.builder.write_file(output_fh, lines)
        output_fh.seek(0)
        output = output_fh.read()
        for line in lines:
            for box in line.word_boxes:
                self.assertIn(box.content, output)
                self.assertIn("{} {} {} {}".format(
                    box.position[0][0], box.position[0][1],
                    box.position[1][0], box.position[1][1],
                ), output)
                self.assertIn(str(box.confidence), output)

    def test_start_line(self):
        position = ((1, 2), (3, 4))
        self.builder.start_line(position)
        self.assertEqual(len(self.builder.lines), 1)
        self.assertListEqual(self.builder.lines,
                             [builders.LineBox([], position)])
        self.builder.start_line(position)
        self.assertEqual(len(self.builder.lines), 1)
        self.assertListEqual(self.builder.lines,
                             [builders.LineBox([], position)])

    def test_add_word_no_line(self):
        box = builders.Box("word", ((1, 2), (3, 4)), 42)
        with self.assertRaises(IndexError):
            self.builder.add_word(box.content, box.position, box.confidence)
        self.assertListEqual(self.builder.lines, [])

    def test_end_line(self):
        before = list(self.builder.lines)
        self.builder.end_line()
        self.assertEqual(self.builder.lines, before)

    def test_get_output(self):
        lines = []
        for l in range(4):
            boxes = []
            for b in range(4):
                word = "word" + str(4*l+b)
                position = ((4*l+b, 0), (0, 0))
                boxes.append(builders.Box(word, position, randint(0, 100)))
            line_position = ((4*l, 4*(l+1)), (4*l+2, 4*(l+1)+2))
            lines.append(builders.LineBox(boxes, line_position))

        for line in lines:
            self.builder.start_line(line.position)
            for word in line.word_boxes:
                self.builder.add_word(word.content, word.position,
                                      word.confidence)
            self.builder.end_line()  # could be useful in future
        output = self.builder.get_output()
        for line, line_expected in zip(output, lines):
            self.assertIsInstance(line, builders.LineBox)
            self.assertEqual(line, line_expected)

    def test_str_method(self):
        self.assertEqual(str(self.builder), "Line boxes")


class TestDigitBuilder(unittest.TestCase):
    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.builder = builders.DigitBuilder()

    def test_init(self):
        self.assertIn("digits", self.builder.tesseract_configs)

    def test_str_method(self):
        self.assertEqual(str(self.builder), "Digits raw text")


class TestDigitLineBoxBuilder(unittest.TestCase):
    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.builder = builders.DigitLineBoxBuilder()

    def test_init(self):
        self.assertIn("digits", self.builder.tesseract_configs)

    def test_str_method(self):
        self.assertEqual(str(self.builder), "Digit line boxes")



import os
import unittest


class BaseTest(unittest.TestCase):
    tool = None

    def _get_file_handle(self, filename):
        return open(os.path.join(os.path.dirname(os.path.abspath(__file__)),
                                 "data", filename),
                    encoding="utf-8")

    def _get_file_content(self, filename):
        with open(os.path.join(os.path.dirname(os.path.abspath(__file__)),
                               "data", filename), encoding="utf-8") as fh:
            content = fh.read()
        return content



import unittest
import xml.dom.minidom

from pyocr import builders


class TestBox(unittest.TestCase):
    """
    These tests ensure the Box features are what they should.
    """
    def setUp(self):
        self.box1 = builders.Box("word1", ((15, 22), (23, 42)))
        self.box1_bis = builders.Box("word1_bis", ((15, 22), (23, 42)))
        self.box2 = builders.Box("word2", ((30, 5), (40, 15)), 95)
        self.box_unicode = builders.Box("\xe9", ((1, 2), (3, 4)))

    def test_init(self):
        self.assertEqual(self.box1.content, "word1")
        self.assertSequenceEqual(self.box1.position, ((15, 22), (23, 42)))
        self.assertEqual(self.box1.confidence, 0)
        self.assertEqual(self.box2.confidence, 95)

    def test_get_xml_tag(self):
        impl = xml.dom.minidom.getDOMImplementation()
        doc = impl.createDocument(None, "root", None)
        tag = self.box1.get_xml_tag(doc)
        self.assertEqual(len(tag.childNodes), 1)
        self.assertEqual(tag.getAttribute("class"), "ocrx_word")
        self.assertEqual(tag.getAttribute("title"),
                         "bbox 15 22 23 42; x_wconf 0")
        self.assertEqual(tag.firstChild.data, "word1")

    def test_str_method(self):
        self.assertEqual(str(self.box1), "word1 15 22 23 42")

    def test_str_unicode(self):
        self.assertEqual(str(self.box_unicode), "\xe9 1 2 3 4")

    def test_box_not_equal_none(self):
        self.assertNotEqual(self.box1, None)

    def test_box_equal(self):
        self.assertEqual(self.box1, self.box1_bis)

    def test_box_not_equal(self):
        self.assertNotEqual(self.box1, self.box2)

    def test_box_lower(self):
        self.assertLess(self.box2, self.box1)
        self.assertLessEqual(self.box2, self.box1)

    def test_box_greater(self):
        self.assertGreater(self.box1, self.box2)
        self.assertGreaterEqual(self.box1, self.box2)

    def test_box_equal_not_box(self):
        self.assertNotEqual(self.box1, 0)
        self.assertNotEqual(self.box1, [])

    def test_hash(self):
        self.assertEqual(hash(self.box1), hash(self.box1_bis))
        self.assertNotEqual(hash(self.box1), hash(self.box2))


class TestLineBox(unittest.TestCase):

    def setUp(self):
        box1 = builders.Box("word1", ((15, 22), (23, 30)))
        box2 = builders.Box("word2", ((25, 23), (30, 32)))
        box3 = builders.Box("word3", ((32, 25), (40, 32)), 95)
        box4 = builders.Box("word4", ((41, 18), (44, 33)), 98)
        box_unicode = builders.Box("\xe9", ((1, 2), (3, 4)), 98)
        self.line1 = builders.LineBox(
            [box1, box2, box3, box4],
            ((14, 15), (45, 33))
        )
        self.line1_bis = builders.LineBox(
            [box1, box2],
            ((14, 15), (45, 33))
        )
        self.line2 = builders.LineBox(
            [box3, box4],
            ((30, 5), (53, 20))
        )
        self.line1_dupl = builders.LineBox(
            [box1, box2, box3, box4],
            ((14, 15), (45, 33))
        )
        self.line_unicode = builders.LineBox(
            [box1, box_unicode],
            ((1, 2), (3, 4))
        )

    def test_init(self):
        self.assertEqual(len(self.line1.word_boxes), 4)
        self.assertSequenceEqual(self.line1.position, ((14, 15), (45, 33)))
        self.assertEqual(self.line1.content, "word1 word2 word3 word4")

    def test_get_xml_tag(self):
        impl = xml.dom.minidom.getDOMImplementation()
        doc = impl.createDocument(None, "root", None)
        tag = self.line1.get_xml_tag(doc)
        self.assertEqual(len(tag.childNodes),
                         2 * len(self.line1.word_boxes) - 1)
        self.assertEqual(tag.getAttribute("class"), "ocr_line")
        self.assertEqual(tag.getAttribute("title"), "bbox 14 15 45 33")
        self.assertEqual(tag.firstChild.firstChild.data, "word1")
        self.assertEqual(tag.lastChild.firstChild.data, "word4")

    def test_line_str(self):
        expected = "[\n"
        for box in self.line1.word_boxes:
            expected += "  " + box.__str__() + "\n"
        expected += "] 14 15 45 33"
        self.assertEqual(str(self.line1), expected)

    def test_str_unicode(self):
        self.assertEqual(
            str(self.line_unicode),
            "[\n  word1 15 22 23 30\n  \xe9 1 2 3 4\n] 1 2 3 4"
        )

    def test_line_not_equal_none(self):
        self.assertNotEqual(self.line1, None)

    def test_box_equal(self):
        self.assertEqual(self.line1, self.line1_bis)

    def test_box_not_equal(self):
        self.assertNotEqual(self.line1, self.line2)

    def test_line_lower(self):
        self.assertLess(self.line2, self.line1)
        self.assertLessEqual(self.line2, self.line1)

    def test_line_greater(self):
        self.assertGreater(self.line1, self.line2)
        self.assertGreaterEqual(self.line1, self.line2)

    def test_hash(self):
        self.assertIsNotNone(hash(self.line1))
        self.assertNotEqual(hash(self.line1), hash(self.line1_bis))
        self.assertNotEqual(hash(self.line1), hash(self.line2))
        self.assertEqual(hash(self.line1), hash(self.line1_dupl))



import unittest

from unittest.mock import patch

import pyocr

from pyocr.util import (
    digits_only,
)


class TestPyOCR(unittest.TestCase):

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    @patch("pyocr.libtesseract.tesseract_raw.is_available")
    @patch("shutil.which")
    def test_available_tools_tesseract4(self, which, is_available, libtess):
        which.return_value = True
        is_available.return_value = True
        libtess.TessVersion.return_value = b"4.0.0"
        self.assertListEqual(
            pyocr.get_available_tools(),
            [
                pyocr.tesseract,
                pyocr.libtesseract,
                pyocr.cuneiform,
            ]
        )

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    @patch("pyocr.libtesseract.tesseract_raw.is_available")
    @patch("shutil.which")
    def test_available_tools_tesseract3(self, which, is_available, libtess):
        which.return_value = True
        is_available.return_value = True
        libtess.TessVersion.return_value = b"3.5.0"
        self.assertListEqual(
            pyocr.get_available_tools(),
            [
                pyocr.tesseract,
                pyocr.libtesseract,
                pyocr.cuneiform,
            ]
        )

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    @patch("pyocr.libtesseract.tesseract_raw.is_available")
    @patch("shutil.which")
    def test_available_tools_tesseract3_0(self, which, is_available, libtess):
        which.return_value = True
        is_available.return_value = True
        libtess.TessVersion.return_value = b"3.0.0"
        self.assertListEqual(
            pyocr.get_available_tools(),
            [
                pyocr.tesseract,
                pyocr.cuneiform,
            ]
        )

    def test_digits_only(self):
        self.assertEqual(digits_only("azer"), 0)
        self.assertEqual(digits_only("10.0.1"), 10)
        self.assertEqual(digits_only("42azer"), 42)
        self.assertEqual(digits_only("qsdf42azer"), 42)
        self.assertEqual(digits_only("v42"), 42)
        self.assertEqual(digits_only("v42x35"), 42)
        self.assertEqual(digits_only("v42x35qsdf"), 42)



import locale
import os

from ctypes import POINTER, cast, c_char_p, c_int
from random import randint
from unittest.mock import patch, call

from PIL import Image

from pyocr import builders
from pyocr import libtesseract
from pyocr.error import TesseractError
from pyocr.libtesseract import tesseract_raw

from .tests_base import BaseTest


class TestLibTesseract(BaseTest):
    """
    These tests make sure the requirements for the tests are met.
    """
    def setUp(self):
        self.handle = randint(0, 2**32-1)
        self.image = Image.new(mode="RGB", size=(1, 1))

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_available(self, libtess):
        libtess.TessVersion.return_value = b"4.0.0"
        self.assertTrue(libtesseract.is_available())
        libtess.TessVersion.assert_called_once_with()

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_not_available(self, raw):
        raw.is_available.return_value = False
        self.assertFalse(libtesseract.is_available())
        raw.is_available.assert_called_once_with()

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_not_available_tesseract3(self, libtess):
        libtess.TessVersion.return_value = b"3.3.0"
        self.assertFalse(libtesseract.is_available())
        libtess.TessVersion.assert_called_once_with()

    @patch("pyocr.libtesseract.get_available_languages")
    def test_can_detect_orientation(self, get_available_languages):
        get_available_languages.return_value = ['eng', 'fra', 'jpn', 'osd']
        self.assertTrue(libtesseract.can_detect_orientation())
        get_available_languages.return_value = ['eng', 'fra', 'jpn']
        self.assertFalse(libtesseract.can_detect_orientation())

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_version(self, libtess):
        libtess.TessVersion.return_value = b"3.05"
        self.assertEqual(libtesseract.get_version(), (3, 5, 0))
        libtess.TessVersion.return_value = b"3.2.1"
        self.assertEqual(libtesseract.get_version(), (3, 2, 1))
        libtess.TessVersion.return_value = b"4.0.0"
        self.assertEqual(libtesseract.get_version(), (4, 0, 0))
        libtess.TessVersion.return_value = b"4.0.0aplha"
        self.assertEqual(libtesseract.get_version(), (4, 0, 0))
        libtess.TessVersion.return_value = b"3.5.1dev1"
        self.assertEqual(libtesseract.get_version(), (3, 5, 1))
        libtess.TessVersion.assert_called_with()
        self.assertEqual(libtess.TessVersion.call_count, 5)

    def test_name(self):
        self.assertEqual(libtesseract.get_name(), "Tesseract (C-API)")

    def test_available_builders(self):
        self.assertEqual(
            libtesseract.get_available_builders(),
            [
                builders.TextBuilder,
                builders.WordBoxBuilder,
                builders.DigitBuilder,
                builders.LineBoxBuilder,
                builders.DigitLineBoxBuilder,
            ]
        )

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_langs(self, libtess):
        libtess.TessBaseAPICreate.return_value = self.handle
        libtess.TessBaseAPIGetAvailableLanguagesAsVector.return_value = [
            b"eng",
            b"fra",
            b"jpn",
            b"osd",
            b""
        ]
        self.assertListEqual(
            libtesseract.get_available_languages(),
            ["eng", "fra", "jpn", "osd"]
        )
        libtess.TessBaseAPICreate.assert_called_once_with()
        self.assertEqual(
            libtess.TessBaseAPIGetAvailableLanguagesAsVector.call_count,
            1
        )
        args = libtess.TessBaseAPIGetAvailableLanguagesAsVector.call_args[0]
        self.assertEqual(len(args), 1)
        self.assertEqual(args[0].value, self.handle)

    def test_tess_box_to_pyocr_box(self):
        box = (0, 1, 2, 3)
        self.assertSequenceEqual(
            libtesseract._tess_box_to_pyocr_box(box),
            ((0, 1), (2, 3))
        )

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_detect_orientation(self, raw):
        raw.init.return_value = self.handle
        expected = {
            "orientation": raw.Orientation.PAGE_RIGHT,
            "confidence": 87,
        }
        raw.detect_os.return_value = expected
        self.assertEqual(
            libtesseract.detect_orientation(self.image),
            {
                "angle": 90,
                "confidence": 87,
            }
        )
        raw.init.assert_called_once_with(lang="osd")
        raw.set_page_seg_mode.assert_called_once_with(
            self.handle, raw.PageSegMode.OSD_ONLY
        )
        raw.set_image.assert_called_once_with(self.handle, self.image)
        raw.detect_os.assert_called_once_with(self.handle)

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_detect_orientation_error(self, raw):
        raw.init.return_value = self.handle
        raw.detect_os.return_value = {"confidence": 0}
        with self.assertRaises(TesseractError) as te:
            libtesseract.detect_orientation(self.image)
        self.assertEqual(te.exception.status, "no script")
        self.assertEqual(te.exception.message, "no script detected")


class TestLibTesseractRaw(BaseTest):

    def setUp(self):
        self.handle = randint(0, 2**32-1)
        self.iterator = randint(0, 2**32-1)
        self.image = Image.new("RGB", size=(1, 1))

    @patch("locale.setlocale")
    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_init_tesseract4(self, libtess, setlocale):
        libtess.TessVersion.return_value = b"4.0.0"
        libtess.TessBaseAPICreate.return_value = self.handle
        for lang in (None, "eng", "fra", "jpn", "osd"):
            api = tesseract_raw.init(lang)
            self.assertEqual(api, self.handle)

            libtess.TessBaseAPICreate.assert_called_once_with()

            self.assertEqual(
                libtess.TessBaseAPIInit3.call_count,
                1
            )
            args = libtess.TessBaseAPIInit3.call_args[0]
            self.assertEqual(len(args), 3)
            self.assertEqual(args[0].value, self.handle)
            self.assertEqual(args[1].value, None)
            self.assertEqual(args[2].value, lang.encode() if lang else None)

            self.assertEqual(
                libtess.TessBaseAPISetVariable.call_count,
                1
            )
            args = libtess.TessBaseAPISetVariable.call_args[0]
            self.assertEqual(len(args), 3)
            self.assertEqual(args[0].value, self.handle)
            self.assertEqual(args[1], b"tessedit_zero_rejection")
            self.assertEqual(args[2], b"F")

            setlocale.assert_called_once_with(locale.LC_ALL, "C")

            libtess.reset_mock()
            setlocale.reset_mock()

    @patch("locale.setlocale")
    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_init_tesseract3(self, libtess, setlocale):
        libtess.TessVersion.return_value = b"3.5.0"
        libtess.TessBaseAPICreate.return_value = self.handle
        for lang in (None, "eng", "fra", "jpn", "osd"):
            api = tesseract_raw.init(lang)
            self.assertEqual(api, self.handle)

            libtess.TessBaseAPICreate.assert_called_once_with()

            self.assertEqual(
                libtess.TessBaseAPIInit3.call_count,
                1
            )
            args = libtess.TessBaseAPIInit3.call_args[0]
            self.assertEqual(len(args), 3)
            self.assertEqual(args[0].value, self.handle)
            self.assertEqual(args[1].value, None)
            self.assertEqual(args[2].value, lang.encode() if lang else None)

            self.assertEqual(
                libtess.TessBaseAPISetVariable.call_count,
                1
            )
            args = libtess.TessBaseAPISetVariable.call_args[0]
            self.assertEqual(len(args), 3)
            self.assertEqual(args[0].value, self.handle)
            self.assertEqual(args[1], b"tessedit_zero_rejection")
            self.assertEqual(args[2], b"F")
            self.assertFalse(setlocale.called)

            libtess.reset_mock()
            setlocale.reset_mock()

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_init_error(self, libtess):
        libtess.TessBaseAPICreate.return_value = self.handle
        libtess.TessBaseAPIInit3.side_effect = Exception(
            "Could not initialize"
        )
        with self.assertRaises(Exception):
            tesseract_raw.init()
        self.assertEqual(
            libtess.TessBaseAPICreate.call_count,
            1
        )
        self.assertEqual(
            libtess.TessBaseAPIDelete.call_count,
            1
        )
        args = libtess.TessBaseAPIDelete.call_args[0]
        self.assertEqual(len(args), 1)
        self.assertEqual(args[0].value, self.handle)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_cleanup(self, libtess):
        tesseract_raw.cleanup(self.handle)
        self.assertEqual(
            libtess.TessBaseAPIDelete.call_count,
            1
        )
        args = libtess.TessBaseAPIDelete.call_args[0]
        self.assertEqual(len(args), 1)
        self.assertEqual(args[0].value, self.handle)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_set_is_numeric(self, libtess):
        for mode in (True, False):
            wl = b"0123456789." if mode else b""
            tesseract_raw.set_is_numeric(self.handle, mode)
            self.assertEqual(
                libtess.TessBaseAPISetVariable.call_count,
                1
            )
            args = libtess.TessBaseAPISetVariable.call_args[0]
            self.assertEqual(len(args), 3)
            self.assertEqual(args[0].value, self.handle)
            self.assertEqual(args[1], b"tessedit_char_whitelist")
            self.assertEqual(args[2], wl)
            libtess.reset_mock()

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_set_debug_file(self, libtess):
        for filename in ("file", b"file"):
            tesseract_raw.set_debug_file(self.handle, filename)
            self.assertEqual(
                libtess.TessBaseAPISetVariable.call_count,
                1
            )
            args = libtess.TessBaseAPISetVariable.call_args[0]
            self.assertEqual(len(args), 3)
            self.assertEqual(args[0].value, self.handle)
            self.assertEqual(args[1], b"debug_file")
            self.assertEqual(args[2], b"file")
            libtess.reset_mock()

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_set_page_seg_mode(self, libtess):
        tesseract_raw.set_page_seg_mode(self.handle, 3)
        self.assertEqual(
            libtess.TessBaseAPISetPageSegMode.call_count,
            1
        )
        args = libtess.TessBaseAPISetPageSegMode.call_args[0]
        self.assertEqual(len(args), 2)
        self.assertEqual(args[0].value, self.handle)
        self.assertEqual(args[1].value, 3)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_init_for_analyse_page(self, libtess):
        tesseract_raw.init_for_analyse_page(self.handle)
        self.assertEqual(
            libtess.TessBaseAPIInitForAnalysePage.call_count,
            1
        )
        args = libtess.TessBaseAPIInitForAnalysePage.call_args[0]
        self.assertEqual(len(args), 1)
        self.assertEqual(args[0].value, self.handle)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_set_image(self, libtess):
        tesseract_raw.set_image(self.handle, self.image)
        self.assertEqual(libtess.TessBaseAPISetImage.call_count, 1)
        args = libtess.TessBaseAPISetImage.call_args[0]
        self.assertEqual(len(args), 6)
        self.assertEqual(args[0].value, self.handle)
        self.assertEqual(args[1], self.image.tobytes("raw", "RGB"))
        self.assertEqual(args[2].value, self.image.width)
        self.assertEqual(args[3].value, self.image.height)
        self.assertEqual(args[4].value, 3)
        self.assertEqual(args[5].value, self.image.width * 3)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_recognize(self, libtess):
        libtess.TessBaseAPIRecognize.return_value = 0
        self.assertEqual(tesseract_raw.recognize(self.handle), 0)
        self.assertEqual(
            libtess.TessBaseAPIRecognize.call_count,
            1
        )
        args = libtess.TessBaseAPIRecognize.call_args[0]
        self.assertEqual(len(args), 2)
        self.assertEqual(args[0].value, self.handle)
        self.assertIsNone(args[1].value)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_analyse_layout(self, libtess):
        layout = randint(0, 2**32-1)
        libtess.TessBaseAPIAnalyseLayout.return_value = layout
        self.assertEqual(tesseract_raw.analyse_layout(self.handle), layout)
        self.assertEqual(
            libtess.TessBaseAPIAnalyseLayout.call_count,
            1
        )
        args = libtess.TessBaseAPIAnalyseLayout.call_args[0]
        self.assertEqual(len(args), 1)
        self.assertEqual(args[0].value, self.handle)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_get_utf8_text(self, libtess):
        text = "Test text for get utf8"
        ptr = c_char_p(text.encode())
        libtess.TessBaseAPIGetUTF8Text.return_value = ptr
        self.assertEqual(tesseract_raw.get_utf8_text(self.handle), text)
        self.assertEqual(
            libtess.TessBaseAPIGetUTF8Text.call_count,
            1
        )
        args = libtess.TessBaseAPIGetUTF8Text.call_args[0]
        self.assertEqual(len(args), 1)
        self.assertEqual(args[0].value, self.handle)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_page_iterator_delete(self, libtess):
        tesseract_raw.page_iterator_delete(self.iterator)
        self.assertEqual(
            libtess.TessPageIteratorDelete.call_count,
            1
        )
        args = libtess.TessPageIteratorDelete.call_args[0]
        self.assertEqual(len(args), 1)
        self.assertEqual(args[0].value, self.iterator)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_page_iterator_next(self, libtess):
        libtess.TessPageIteratorNext.return_value = self.iterator + 1
        self.assertEqual(tesseract_raw.page_iterator_next(
            self.iterator,
            tesseract_raw.PageIteratorLevel.WORD
        ), self.iterator + 1)
        self.assertEqual(
            libtess.TessPageIteratorNext.call_count,
            1
        )
        args = libtess.TessPageIteratorNext.call_args[0]
        self.assertEqual(len(args), 2)
        self.assertEqual(args[0].value, self.iterator)
        self.assertEqual(args[1], tesseract_raw.PageIteratorLevel.WORD)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_page_iterator_beginning(self, libtess):
        libtess.TessPageIteratorIsAtBeginningOf.return_value = True
        self.assertTrue(tesseract_raw.page_iterator_is_at_beginning_of(
            self.iterator,
            tesseract_raw.PageIteratorLevel.WORD
        ))
        self.assertEqual(
            libtess.TessPageIteratorIsAtBeginningOf.call_count,
            1
        )
        args = libtess.TessPageIteratorIsAtBeginningOf.call_args[0]
        self.assertEqual(len(args), 2)
        self.assertEqual(args[0].value, self.iterator)
        self.assertEqual(args[1], tesseract_raw.PageIteratorLevel.WORD)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_page_iterator_final(self, libtess):
        element = randint(0, 2**32-1)
        libtess.TessPageIteratorIsAtFinalElement.return_value = True
        self.assertTrue(tesseract_raw.page_iterator_is_at_final_element(
            self.iterator,
            tesseract_raw.PageIteratorLevel.WORD,
            element
        ))
        self.assertEqual(
            libtess.TessPageIteratorIsAtFinalElement.call_count,
            1
        )
        args = libtess.TessPageIteratorIsAtFinalElement.call_args[0]
        self.assertEqual(len(args), 3)
        self.assertEqual(args[0].value, self.iterator)
        self.assertEqual(args[1], tesseract_raw.PageIteratorLevel.WORD)
        self.assertEqual(args[2], element)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_page_iterator_block_type(self, libtess):
        flowing = tesseract_raw.PolyBlockType.FLOWING_TEXT
        libtess.TessPageIteratorBlockType.return_value = flowing
        self.assertEqual(tesseract_raw.page_iterator_block_type(self.iterator),
                         tesseract_raw.PolyBlockType.FLOWING_TEXT)
        self.assertEqual(
            libtess.TessPageIteratorBlockType.call_count,
            1
        )
        args = libtess.TessPageIteratorBlockType.call_args[0]
        self.assertEqual(len(args), 1)
        self.assertEqual(args[0].value, self.iterator)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_page_iterator_box(self, libtess):
        for res in (True, False):
            libtess.TessPageIteratorBoundingBox.return_value = res
            level = tesseract_raw.PageIteratorLevel.WORD
            result, box = tesseract_raw.page_iterator_bounding_box(
                self.iterator, level
            )
            self.assertEqual(result, res)
            self.assertSequenceEqual(box, (0, 0, 0, 0))
            self.assertEqual(
                libtess.TessPageIteratorBoundingBox.call_count,
                1
            )
            args = libtess.TessPageIteratorBoundingBox.call_args[0]
            self.assertEqual(len(args), 6)
            self.assertEqual(args[0].value, self.iterator)
            self.assertEqual(args[1], level)
            self.assertEqual(cast(args[2], POINTER(c_int)).contents.value, 0)
            self.assertEqual(cast(args[3], POINTER(c_int)).contents.value, 0)
            self.assertEqual(cast(args[4], POINTER(c_int)).contents.value, 0)
            self.assertEqual(cast(args[5], POINTER(c_int)).contents.value, 0)

            libtess.reset_mock()

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_page_iterator_orientation(self, libtess):
        expected = {
            "orientation": 0,
            "writing_direction": 0,
            "textline_order": 0,
            "deskew_angle": 0,
        }
        self.assertEqual(
            tesseract_raw.page_iterator_orientation(self.iterator),
            expected
        )
        self.assertEqual(
            libtess.TessPageIteratorOrientation.call_count,
            1
        )
        args = libtess.TessPageIteratorOrientation.call_args[0]
        self.assertEqual(len(args), 5)
        self.assertEqual(args[0].value, self.iterator)
        self.assertEqual(cast(args[1], POINTER(c_int)).contents.value, 0)
        self.assertEqual(cast(args[2], POINTER(c_int)).contents.value, 0)
        self.assertEqual(cast(args[3], POINTER(c_int)).contents.value, 0)
        self.assertEqual(cast(args[4], POINTER(c_int)).contents.value, 0)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_get_iterator(self, libtess):
        libtess.TessBaseAPIGetIterator.return_value = self.iterator
        self.assertEqual(tesseract_raw.get_iterator(self.handle),
                         self.iterator)
        self.assertEqual(
            libtess.TessBaseAPIGetIterator.call_count,
            1
        )
        args = libtess.TessBaseAPIGetIterator.call_args[0]
        self.assertEqual(len(args), 1)
        self.assertEqual(args[0].value, self.handle)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_result_iterator_page(self, libtess):
        libtess.TessResultIteratorGetPageIterator.return_value = self.iterator
        self.assertEqual(
            tesseract_raw.result_iterator_get_page_iterator(self.iterator),
            self.iterator
        )
        self.assertEqual(
            libtess.TessResultIteratorGetPageIterator.call_count,
            1
        )
        args = libtess.TessResultIteratorGetPageIterator.call_args[0]
        self.assertEqual(len(args), 1)
        self.assertEqual(args[0].value, self.iterator)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_result_iterator_text(self, libtess):
        level = tesseract_raw.PageIteratorLevel.WORD
        text = "Test text for get utf8"
        ptr = c_char_p(text.encode())
        libtess.TessResultIteratorGetUTF8Text.return_value = ptr
        self.assertEqual(
            tesseract_raw.result_iterator_get_utf8_text(self.iterator, level),
            text
        )
        self.assertEqual(
            libtess.TessResultIteratorGetUTF8Text.call_count,
            1
        )
        args = libtess.TessResultIteratorGetUTF8Text.call_args[0]
        self.assertEqual(len(args), 2)
        self.assertEqual(args[0].value, self.iterator)
        self.assertEqual(args[1], level)
        libtess.TessDeleteText.assert_called_once_with(ptr)

        libtess.reset_mock()

        libtess.TessResultIteratorGetUTF8Text.return_value = None
        self.assertIsNone(
            tesseract_raw.result_iterator_get_utf8_text(self.iterator, level)
        )
        self.assertEqual(
            libtess.TessResultIteratorGetUTF8Text.call_count,
            1
        )
        args = libtess.TessResultIteratorGetUTF8Text.call_args[0]
        self.assertEqual(len(args), 2)
        self.assertEqual(args[0].value, self.iterator)
        self.assertEqual(args[1], level)
        self.assertFalse(libtess.TessDeleteText.called)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_result_iterator_confidence(self, libtess):
        level = tesseract_raw.PageIteratorLevel.WORD
        libtess.TessResultIteratorConfidence.return_value = 95
        self.assertEqual(
            tesseract_raw.result_iterator_get_confidence(self.iterator, level),
            95
        )
        self.assertEqual(
            libtess.TessResultIteratorConfidence.call_count,
            1
        )
        args = libtess.TessResultIteratorConfidence.call_args[0]
        self.assertEqual(len(args), 2)
        self.assertEqual(args[0].value, self.iterator)
        self.assertEqual(args[1], level)

        libtess.reset_mock()

        libtess.TessResultIteratorConfidence.return_value = None
        self.assertIsNone(
            tesseract_raw.result_iterator_get_confidence(self.iterator, level)
        )
        self.assertEqual(
            libtess.TessResultIteratorConfidence.call_count,
            1
        )
        args = libtess.TessResultIteratorConfidence.call_args[0]
        self.assertEqual(len(args), 2)
        self.assertEqual(args[0].value, self.iterator)
        self.assertEqual(args[1], level)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_detect_os(self, libtess):
        libtess.TessBaseAPIDetectOrientationScript.return_value = True
        self.assertEqual(
            tesseract_raw.detect_os(self.handle),
            {
                "orientation": 0,
                "confidence": 0,
            }
        )
        self.assertEqual(
            libtess.TessBaseAPIDetectOrientationScript.call_count,
            1
        )
        args = libtess.TessBaseAPIDetectOrientationScript.call_args[0]
        self.assertEqual(len(args), 5)
        self.assertEqual(args[0].value, self.handle)
        self.assertEqual(cast(args[1], POINTER(c_int)).contents.value, 0)
        self.assertEqual(cast(args[2], POINTER(c_int)).contents.value, 0)
        self.assertIsNone(args[3])
        self.assertIsNone(args[4])

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_detect_os_error(self, libtess):
        libtess.TessBaseAPIDetectOrientationScript.return_value = False
        with self.assertRaises(TesseractError) as te:
            tesseract_raw.detect_os(self.handle)
        self.assertEqual(te.exception.status, "detect_orientation failed")
        self.assertEqual(te.exception.message,
                         "TessBaseAPIDetectOrientationScript() failed")
        self.assertEqual(
            libtess.TessBaseAPIDetectOrientationScript.call_count,
            1
        )
        args = libtess.TessBaseAPIDetectOrientationScript.call_args[0]
        self.assertEqual(len(args), 5)
        self.assertEqual(args[0].value, self.handle)
        self.assertEqual(cast(args[1], POINTER(c_int)).contents.value, 0)
        self.assertEqual(cast(args[2], POINTER(c_int)).contents.value, 0)
        self.assertIsNone(args[3])
        self.assertIsNone(args[4])

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_detect_os_old(self, libtess):
        del libtess.TessBaseAPIDetectOrientationScript
        libtess.TessBaseAPIDetectOS.return_value = True
        self.assertEqual(
            tesseract_raw.detect_os(self.handle),
            {
                "orientation": 0,
                "confidence": 0,
            }
        )
        self.assertEqual(
            libtess.TessBaseAPIDetectOS.call_count,
            1
        )
        args = libtess.TessBaseAPIDetectOS.call_args[0]
        self.assertEqual(len(args), 2)
        self.assertEqual(args[0].value, self.handle)
        self.assertIsInstance(
            cast(args[1], POINTER(tesseract_raw.OSResults)).contents,
            tesseract_raw.OSResults
        )

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_detect_os_old_error(self, libtess):
        del libtess.TessBaseAPIDetectOrientationScript
        libtess.TessBaseAPIDetectOS.return_value = False
        with self.assertRaises(TesseractError) as te:
            tesseract_raw.detect_os(self.handle)
        self.assertEqual(te.exception.status, "detect_orientation failed")
        self.assertEqual(te.exception.message,
                         "TessBaseAPIDetectOS() failed")
        self.assertEqual(
            libtess.TessBaseAPIDetectOS.call_count,
            1
        )
        args = libtess.TessBaseAPIDetectOS.call_args[0]
        self.assertEqual(len(args), 2)
        self.assertEqual(args[0].value, self.handle)
        self.assertIsInstance(
            cast(args[1], POINTER(tesseract_raw.OSResults)).contents,
            tesseract_raw.OSResults
        )

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_set_input_name(self, libtess):
        input_file = "file"
        tesseract_raw.set_input_name(self.handle, input_file)
        self.assertEqual(
            libtess.TessBaseAPISetInputName.call_count,
            1
        )
        args = libtess.TessBaseAPISetInputName.call_args[0]
        self.assertEqual(len(args), 2)
        self.assertEqual(args[0].value, self.handle)
        self.assertEqual(args[1], input_file.encode())

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_init_pdf(self, libtess):
        output_file = "file"
        renderer = randint(0, 2**32-1)
        tessdata_dir = "/path/to/tess/data"
        libtess.TessBaseAPIGetDatapath.return_value = tessdata_dir
        libtess.TessPDFRendererCreate.return_value = renderer
        self.assertEqual(
            tesseract_raw.init_pdf_renderer(self.handle, output_file, True),
            renderer
        )
        libtess.TessBaseAPIGetDatapath.assert_called_once_with(self.handle)
        self.assertEqual(
            libtess.TessPDFRendererCreate.call_count,
            1
        )
        args = libtess.TessPDFRendererCreate.call_args[0]
        self.assertEqual(len(args), 3)
        self.assertEqual(args[0], output_file.encode())
        self.assertEqual(args[1], tessdata_dir)
        self.assertEqual(args[2].value, True)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_begin_doc(self, libtess):
        renderer = randint(0, 2**32-1)
        doc_name = "doc"
        tesseract_raw.begin_document(renderer, doc_name)
        self.assertEqual(
            libtess.TessResultRendererBeginDocument.call_count,
            1
        )
        args = libtess.TessResultRendererBeginDocument.call_args[0]
        self.assertEqual(len(args), 2)
        self.assertEqual(args[0].value, renderer)
        self.assertEqual(args[1], doc_name.encode())

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_add_renderer_image(self, libtess):
        renderer = randint(0, 2**32-1)
        tesseract_raw.add_renderer_image(self.handle, renderer)
        self.assertEqual(
            libtess.TessResultRendererAddImage.call_count,
            1
        )
        args = libtess.TessResultRendererAddImage.call_args[0]
        self.assertEqual(len(args), 2)
        self.assertEqual(args[0].value, renderer)
        self.assertEqual(args[1].value, self.handle)

    @patch("pyocr.libtesseract.tesseract_raw.g_libtesseract")
    def test_end_doc(self, libtess):
        renderer = randint(0, 2**32-1)
        tesseract_raw.end_document(renderer)
        self.assertEqual(
            libtess.TessResultRendererEndDocument.call_count,
            1
        )
        args = libtess.TessResultRendererEndDocument.call_args[0]
        self.assertEqual(len(args), 1)
        self.assertEqual(args[0].value, renderer)


class TestLibTesseractText(BaseTest):

    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.image = Image.new(mode="RGB", size=(1, 1))
        self.builder = builders.TextBuilder()
        self.handle = randint(0, 2**32-1)
        self.iterator = randint(0, 2**32-1)

    @patch("pyocr.tesseract.get_version")
    @patch("pyocr.libtesseract.tesseract_raw")
    def test_image_to_string_defaults_to_text_buidler(self, raw, get_version):
        get_version.return_value = (4, 0, 0)
        raw.init.return_value = self.handle
        raw.get_iterator.return_value = self.iterator
        raw.result_iterator_get_page_iterator.return_value = self.iterator
        raw.get_available_languages.return_value = ["eng", "fra", "jpn", "osd"]
        raw.page_iterator_next.side_effect = (True, True, False)
        raw.page_iterator_bounding_box.return_value = (True, (0, 0, 0, 0))
        raw.result_iterator_get_utf8_text.side_effect = ("word1", "word2",
                                                         "word3")
        raw.page_iterator_is_at_beginning_of.side_effect = (True, False, False)
        raw.page_iterator_is_at_final_element.side_effect = (False, False,
                                                             True)

        self.assertEqual(
            libtesseract.image_to_string(self.image),
            "word1 word2 word3"
        )

        raw.init.assert_called_once_with(lang=None)
        raw.get_available_languages.assert_called_once_with(self.handle)
        raw.set_page_seg_mode.assert_called_once_with(
            self.handle, self.builder.tesseract_layout)
        raw.set_debug_file.assert_called_once_with(self.handle, os.devnull)
        raw.set_image.assert_called_once_with(self.handle, self.image)
        self.assertFalse(raw.set_is_numeric.called)
        raw.recognize.assert_called_once_with(self.handle)
        raw.get_iterator.assert_called_once_with(self.handle)
        raw.result_iterator_get_page_iterator.assert_called_once_with(
            self.iterator
        )
        self.assertEqual(raw.page_iterator_is_at_beginning_of.call_count, 3)
        raw.page_iterator_is_at_beginning_of.assert_called_with(
            self.iterator, raw.PageIteratorLevel.TEXTLINE)

        # called first at beginning and three times for each word
        self.assertEqual(raw.page_iterator_bounding_box.call_count, 4)
        self.assertListEqual(
            raw.page_iterator_bounding_box.call_args_list,
            [
                call(self.iterator, raw.PageIteratorLevel.TEXTLINE),
                call(self.iterator, raw.PageIteratorLevel.WORD),
                call(self.iterator, raw.PageIteratorLevel.WORD),
                call(self.iterator, raw.PageIteratorLevel.WORD),
            ]
        )
        self.assertEqual(raw.page_iterator_is_at_final_element.call_count, 3)
        raw.page_iterator_is_at_final_element.assert_called_with(
            self.iterator, raw.PageIteratorLevel.TEXTLINE,
            raw.PageIteratorLevel.WORD
        )
        self.assertEqual(raw.result_iterator_get_utf8_text.call_count, 3)
        raw.result_iterator_get_utf8_text.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        self.assertEqual(raw.result_iterator_get_confidence.call_count, 3)
        raw.result_iterator_get_confidence.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        self.assertEqual(raw.page_iterator_next.call_count, 3)
        raw.page_iterator_next.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        raw.cleanup.assert_called_once_with(self.handle)

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_lang(self, raw):
        raw.init.return_value = self.handle
        raw.get_iterator.return_value = self.iterator
        raw.result_iterator_get_page_iterator.return_value = self.iterator
        raw.get_available_languages.return_value = ["eng", "fra", "jpn", "osd"]
        raw.page_iterator_next.side_effect = (True, True, False)
        raw.page_iterator_bounding_box.return_value = (True, (0, 0, 0, 0))
        raw.result_iterator_get_utf8_text.side_effect = ("word1", "word2",
                                                         "word3")
        raw.page_iterator_is_at_beginning_of.side_effect = (True, False, False)
        raw.page_iterator_is_at_final_element.side_effect = (False, False,
                                                             True)

        self.assertEqual(
            libtesseract.image_to_string(self.image, lang="eng",
                                         builder=self.builder),
            "word1 word2 word3"
        )

        raw.init.assert_called_once_with(lang="eng")
        raw.get_available_languages.assert_called_once_with(self.handle)
        raw.set_page_seg_mode.assert_called_once_with(
            self.handle, self.builder.tesseract_layout)
        raw.set_debug_file.assert_called_once_with(self.handle, os.devnull)
        raw.set_image.assert_called_once_with(self.handle, self.image)
        self.assertFalse(raw.set_is_numeric.called)
        raw.recognize.assert_called_once_with(self.handle)
        raw.get_iterator.assert_called_once_with(self.handle)
        raw.result_iterator_get_page_iterator.assert_called_once_with(
            self.iterator
        )
        self.assertEqual(raw.page_iterator_is_at_beginning_of.call_count, 3)
        raw.page_iterator_is_at_beginning_of.assert_called_with(
            self.iterator, raw.PageIteratorLevel.TEXTLINE)

        # called first at beginning and three times for each word
        self.assertEqual(raw.page_iterator_bounding_box.call_count, 4)
        self.assertListEqual(
            raw.page_iterator_bounding_box.call_args_list,
            [
                call(self.iterator, raw.PageIteratorLevel.TEXTLINE),
                call(self.iterator, raw.PageIteratorLevel.WORD),
                call(self.iterator, raw.PageIteratorLevel.WORD),
                call(self.iterator, raw.PageIteratorLevel.WORD),
            ]
        )
        self.assertEqual(raw.page_iterator_is_at_final_element.call_count, 3)
        raw.page_iterator_is_at_final_element.assert_called_with(
            self.iterator, raw.PageIteratorLevel.TEXTLINE,
            raw.PageIteratorLevel.WORD
        )
        self.assertEqual(raw.result_iterator_get_utf8_text.call_count, 3)
        raw.result_iterator_get_utf8_text.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        self.assertEqual(raw.result_iterator_get_confidence.call_count, 3)
        raw.result_iterator_get_confidence.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        self.assertEqual(raw.page_iterator_next.call_count, 3)
        raw.page_iterator_next.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        raw.cleanup.assert_called_once_with(self.handle)

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_lang_error(self, raw):
        raw.init.return_value = self.handle
        raw.get_available_languages.return_value = ["eng", "jpn", "osd"]

        with self.assertRaises(TesseractError) as te:
            libtesseract.image_to_string(self.image, lang="fra",
                                         builder=self.builder)
        self.assertEqual(te.exception.status, "no lang")
        self.assertEqual(te.exception.message, "language fra is not available")

        raw.init.assert_called_once_with(lang="fra")
        raw.get_available_languages.assert_called_once_with(self.handle)
        raw.cleanup.assert_called_once_with(self.handle)

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_text(self, raw):
        raw.init.return_value = self.handle
        raw.get_iterator.return_value = self.iterator
        raw.result_iterator_get_page_iterator.return_value = self.iterator
        raw.get_available_languages.return_value = ["eng", "fra", "jpn", "osd"]
        raw.page_iterator_next.side_effect = (True, True, True, False)
        raw.page_iterator_bounding_box.return_value = (True, (0, 0, 0, 0))
        raw.result_iterator_get_utf8_text.side_effect = ("word1", "word2",
                                                         None, "word3")
        raw.page_iterator_is_at_beginning_of.side_effect = (True, False,
                                                            False, False)
        raw.page_iterator_is_at_final_element.side_effect = (False, False,
                                                             False, True)

        self.assertEqual(
            libtesseract.image_to_string(self.image, builder=self.builder),
            "word1 word2 word3"
        )

        raw.init.assert_called_once_with(lang=None)
        raw.get_available_languages.assert_called_once_with(self.handle)
        raw.set_page_seg_mode.assert_called_once_with(
            self.handle, self.builder.tesseract_layout)
        raw.set_debug_file.assert_called_once_with(self.handle, os.devnull)
        raw.set_image.assert_called_once_with(self.handle, self.image)
        self.assertFalse(raw.set_is_numeric.called)
        raw.recognize.assert_called_once_with(self.handle)
        raw.get_iterator.assert_called_once_with(self.handle)
        raw.result_iterator_get_page_iterator.assert_called_once_with(
            self.iterator
        )
        self.assertEqual(raw.page_iterator_is_at_beginning_of.call_count, 4)
        raw.page_iterator_is_at_beginning_of.assert_called_with(
            self.iterator, raw.PageIteratorLevel.TEXTLINE)

        # called first at beginning and three times for each word
        self.assertEqual(raw.page_iterator_bounding_box.call_count, 4)
        self.assertListEqual(
            raw.page_iterator_bounding_box.call_args_list,
            [
                call(self.iterator, raw.PageIteratorLevel.TEXTLINE),
                call(self.iterator, raw.PageIteratorLevel.WORD),
                call(self.iterator, raw.PageIteratorLevel.WORD),
                call(self.iterator, raw.PageIteratorLevel.WORD),
            ]
        )
        self.assertEqual(raw.page_iterator_is_at_final_element.call_count, 4)
        raw.page_iterator_is_at_final_element.assert_called_with(
            self.iterator, raw.PageIteratorLevel.TEXTLINE,
            raw.PageIteratorLevel.WORD
        )
        self.assertEqual(raw.result_iterator_get_utf8_text.call_count, 4)
        raw.result_iterator_get_utf8_text.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        self.assertEqual(raw.result_iterator_get_confidence.call_count, 4)
        raw.result_iterator_get_confidence.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        self.assertEqual(raw.page_iterator_next.call_count, 4)
        raw.page_iterator_next.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        raw.cleanup.assert_called_once_with(self.handle)

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_text_error(self, raw):
        raw.init.return_value = self.handle
        raw.get_iterator.return_value = None
        raw.result_iterator_get_page_iterator.return_value = self.iterator
        raw.get_available_languages.return_value = ["eng", "fra", "jpn", "osd"]
        raw.page_iterator_next.side_effect = (True, True, False)
        raw.page_iterator_bounding_box.return_value = (True, (0, 0, 0, 0))
        raw.result_iterator_get_utf8_text.side_effect = ("word1", "word2",
                                                         "word3")
        raw.page_iterator_is_at_beginning_of.side_effect = (True, False, False)
        raw.page_iterator_is_at_final_element.side_effect = (False, False,
                                                             True)

        with self.assertRaises(TesseractError) as te:
            libtesseract.image_to_string(self.image, builder=self.builder)
        self.assertEqual(te.exception.status, "no script")
        self.assertEqual(te.exception.message, "no script detected")

        raw.init.assert_called_once_with(lang=None)
        raw.get_available_languages.assert_called_once_with(self.handle)
        raw.set_page_seg_mode.assert_called_once_with(
            self.handle, self.builder.tesseract_layout)
        raw.set_debug_file.assert_called_once_with(self.handle, os.devnull)
        raw.set_image.assert_called_once_with(self.handle, self.image)
        self.assertFalse(raw.set_is_numeric.called)
        raw.recognize.assert_called_once_with(self.handle)
        raw.get_iterator.assert_called_once_with(self.handle)


class TestLibTesseractDigits(BaseTest):

    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.builder = builders.DigitBuilder()
        self.image = Image.new(mode="RGB", size=(1, 1))
        self.handle = randint(0, 2**32-1)
        self.iterator = randint(0, 2**32-1)

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_digits(self, raw):
        raw.init.return_value = self.handle
        raw.get_iterator.return_value = self.iterator
        raw.result_iterator_get_page_iterator.return_value = self.iterator
        raw.get_available_languages.return_value = ["eng", "fra", "jpn", "osd"]
        raw.page_iterator_next.side_effect = (True, True, False)
        raw.page_iterator_bounding_box.return_value = (True, (0, 0, 0, 0))
        raw.result_iterator_get_utf8_text.side_effect = ("1", "2", "42")
        raw.page_iterator_is_at_beginning_of.side_effect = (True, False, False)
        raw.page_iterator_is_at_final_element.side_effect = (False, False,
                                                             True)

        self.assertEqual(
            libtesseract.image_to_string(self.image, builder=self.builder),
            "1 2 42"
        )

        raw.init.assert_called_once_with(lang=None)
        raw.get_available_languages.assert_called_once_with(self.handle)
        raw.set_page_seg_mode.assert_called_once_with(
            self.handle, self.builder.tesseract_layout)
        raw.set_debug_file.assert_called_once_with(self.handle, os.devnull)
        raw.set_image.assert_called_once_with(self.handle, self.image)
        raw.set_is_numeric.assert_called_once_with(self.handle, True)
        raw.recognize.assert_called_once_with(self.handle)
        raw.get_iterator.assert_called_once_with(self.handle)
        raw.result_iterator_get_page_iterator.assert_called_once_with(
            self.iterator
        )
        self.assertEqual(raw.page_iterator_is_at_beginning_of.call_count, 3)
        raw.page_iterator_is_at_beginning_of.assert_called_with(
            self.iterator, raw.PageIteratorLevel.TEXTLINE)

        # called first at beginning and three times for each word
        self.assertEqual(raw.page_iterator_bounding_box.call_count, 4)
        self.assertListEqual(
            raw.page_iterator_bounding_box.call_args_list,
            [
                call(self.iterator, raw.PageIteratorLevel.TEXTLINE),
                call(self.iterator, raw.PageIteratorLevel.WORD),
                call(self.iterator, raw.PageIteratorLevel.WORD),
                call(self.iterator, raw.PageIteratorLevel.WORD),
            ]
        )
        self.assertEqual(raw.page_iterator_is_at_final_element.call_count, 3)
        raw.page_iterator_is_at_final_element.assert_called_with(
            self.iterator, raw.PageIteratorLevel.TEXTLINE,
            raw.PageIteratorLevel.WORD
        )
        self.assertEqual(raw.result_iterator_get_utf8_text.call_count, 3)
        raw.result_iterator_get_utf8_text.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        self.assertEqual(raw.result_iterator_get_confidence.call_count, 3)
        raw.result_iterator_get_confidence.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        self.assertEqual(raw.page_iterator_next.call_count, 3)
        raw.page_iterator_next.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        raw.cleanup.assert_called_once_with(self.handle)


class TestLibTesseractWordBox(BaseTest):

    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.builder = builders.WordBoxBuilder()
        self.image = Image.new("RGB", size=(1, 1))
        self.handle = randint(0, 2**32-1)
        self.iterator = randint(0, 2**32-1)

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_word(self, raw):
        raw.init.return_value = self.handle
        raw.get_iterator.return_value = self.iterator
        raw.result_iterator_get_page_iterator.return_value = self.iterator
        raw.get_available_languages.return_value = ["eng", "fra", "jpn", "osd"]
        raw.page_iterator_next.side_effect = (True, True, False)
        raw.page_iterator_bounding_box.return_value = (True, (0, 0, 0, 0))
        raw.result_iterator_get_utf8_text.side_effect = ("word1", "word2",
                                                         "word3")
        raw.page_iterator_is_at_beginning_of.side_effect = (True, False, False)
        raw.page_iterator_is_at_final_element.side_effect = (False, False,
                                                             True)

        self.assertListEqual(
            libtesseract.image_to_string(self.image, builder=self.builder),
            [
                builders.Box("word1", ((0, 0), (0, 0))),
                builders.Box("word2", ((0, 0), (0, 0))),
                builders.Box("word3", ((0, 0), (0, 0))),
            ]
        )

        raw.init.assert_called_once_with(lang=None)
        raw.get_available_languages.assert_called_once_with(self.handle)
        raw.set_page_seg_mode.assert_called_once_with(
            self.handle, self.builder.tesseract_layout)
        raw.set_debug_file.assert_called_once_with(self.handle, os.devnull)
        raw.set_image.assert_called_once_with(self.handle, self.image)
        self.assertFalse(raw.set_is_numeric.called)
        raw.recognize.assert_called_once_with(self.handle)
        raw.get_iterator.assert_called_once_with(self.handle)
        raw.result_iterator_get_page_iterator.assert_called_once_with(
            self.iterator
        )
        self.assertEqual(raw.page_iterator_is_at_beginning_of.call_count, 3)
        raw.page_iterator_is_at_beginning_of.assert_called_with(
            self.iterator, raw.PageIteratorLevel.TEXTLINE)

        # called first at beginning and three times for each word
        self.assertEqual(raw.page_iterator_bounding_box.call_count, 4)
        self.assertListEqual(
            raw.page_iterator_bounding_box.call_args_list,
            [
                call(self.iterator, raw.PageIteratorLevel.TEXTLINE),
                call(self.iterator, raw.PageIteratorLevel.WORD),
                call(self.iterator, raw.PageIteratorLevel.WORD),
                call(self.iterator, raw.PageIteratorLevel.WORD),
            ]
        )
        self.assertEqual(raw.page_iterator_is_at_final_element.call_count, 3)
        raw.page_iterator_is_at_final_element.assert_called_with(
            self.iterator, raw.PageIteratorLevel.TEXTLINE,
            raw.PageIteratorLevel.WORD
        )
        self.assertEqual(raw.result_iterator_get_utf8_text.call_count, 3)
        raw.result_iterator_get_utf8_text.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        self.assertEqual(raw.result_iterator_get_confidence.call_count, 3)
        raw.result_iterator_get_confidence.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        self.assertEqual(raw.page_iterator_next.call_count, 3)
        raw.page_iterator_next.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        raw.cleanup.assert_called_once_with(self.handle)

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_word_error(self, raw):
        raw.init.return_value = self.handle
        raw.get_iterator.return_value = None
        raw.result_iterator_get_page_iterator.return_value = self.iterator
        raw.get_available_languages.return_value = ["eng", "fra", "jpn", "osd"]
        raw.page_iterator_next.side_effect = (True, True, False)
        raw.page_iterator_bounding_box.return_value = (True, (0, 0, 0, 0))
        raw.result_iterator_get_utf8_text.side_effect = ("word1", "word2",
                                                         "word3")
        raw.page_iterator_is_at_beginning_of.side_effect = (True, False, False)
        raw.page_iterator_is_at_final_element.side_effect = (False, False,
                                                             True)

        with self.assertRaises(TesseractError) as te:
            libtesseract.image_to_string(self.image, builder=self.builder)
        self.assertEqual(te.exception.status, "no script")
        self.assertEqual(te.exception.message, "no script detected")

        raw.init.assert_called_once_with(lang=None)
        raw.get_available_languages.assert_called_once_with(self.handle)
        raw.set_page_seg_mode.assert_called_once_with(
            self.handle, self.builder.tesseract_layout)
        raw.set_debug_file.assert_called_once_with(self.handle, os.devnull)
        raw.set_image.assert_called_once_with(self.handle, self.image)
        self.assertFalse(raw.set_is_numeric.called)
        raw.recognize.assert_called_once_with(self.handle)
        raw.get_iterator.assert_called_once_with(self.handle)


class TestLibTesseractLineBox(BaseTest):

    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.image = Image.new(mode="RGB", size=(1, 1))
        self.builder = builders.LineBoxBuilder()
        self.handle = randint(0, 2**32-1)
        self.iterator = randint(0, 2**32-1)

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_line(self, raw):
        raw.init.return_value = self.handle
        raw.get_iterator.return_value = self.iterator
        raw.result_iterator_get_page_iterator.return_value = self.iterator
        raw.get_available_languages.return_value = ["eng", "fra", "jpn", "osd"]
        raw.page_iterator_next.side_effect = (True, True, False)
        raw.page_iterator_bounding_box.return_value = (True, (0, 0, 0, 0))
        raw.result_iterator_get_utf8_text.side_effect = ("word1", "word2",
                                                         "word3")
        raw.page_iterator_is_at_beginning_of.side_effect = (True, False, False)
        raw.page_iterator_is_at_final_element.side_effect = (False, False,
                                                             True)

        self.assertListEqual(
            libtesseract.image_to_string(self.image, builder=self.builder),
            [
                builders.LineBox([
                    builders.Box("word1", ((0, 0), (0, 0))),
                    builders.Box("word2", ((0, 0), (0, 0))),
                    builders.Box("word3", ((0, 0), (0, 0))),
                ], ((0, 0), (0, 0)))
            ]
        )

        raw.init.assert_called_once_with(lang=None)
        raw.get_available_languages.assert_called_once_with(self.handle)
        raw.set_page_seg_mode.assert_called_once_with(
            self.handle, self.builder.tesseract_layout)
        raw.set_debug_file.assert_called_once_with(self.handle, os.devnull)
        raw.set_image.assert_called_once_with(self.handle, self.image)
        self.assertFalse(raw.set_is_numeric.called)
        raw.recognize.assert_called_once_with(self.handle)
        raw.get_iterator.assert_called_once_with(self.handle)
        raw.result_iterator_get_page_iterator.assert_called_once_with(
            self.iterator
        )
        self.assertEqual(raw.page_iterator_is_at_beginning_of.call_count, 3)
        raw.page_iterator_is_at_beginning_of.assert_called_with(
            self.iterator, raw.PageIteratorLevel.TEXTLINE)

        # called first at beginning and three times for each word
        self.assertEqual(raw.page_iterator_bounding_box.call_count, 4)
        self.assertListEqual(
            raw.page_iterator_bounding_box.call_args_list,
            [
                call(self.iterator, raw.PageIteratorLevel.TEXTLINE),
                call(self.iterator, raw.PageIteratorLevel.WORD),
                call(self.iterator, raw.PageIteratorLevel.WORD),
                call(self.iterator, raw.PageIteratorLevel.WORD),
            ]
        )
        self.assertEqual(raw.page_iterator_is_at_final_element.call_count, 3)
        raw.page_iterator_is_at_final_element.assert_called_with(
            self.iterator, raw.PageIteratorLevel.TEXTLINE,
            raw.PageIteratorLevel.WORD
        )
        self.assertEqual(raw.result_iterator_get_utf8_text.call_count, 3)
        raw.result_iterator_get_utf8_text.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        self.assertEqual(raw.result_iterator_get_confidence.call_count, 3)
        raw.result_iterator_get_confidence.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        self.assertEqual(raw.page_iterator_next.call_count, 3)
        raw.page_iterator_next.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        raw.cleanup.assert_called_once_with(self.handle)

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_line_error(self, raw):
        raw.init.return_value = self.handle
        raw.get_iterator.return_value = None
        raw.result_iterator_get_page_iterator.return_value = self.iterator
        raw.get_available_languages.return_value = ["eng", "fra", "jpn", "osd"]
        raw.page_iterator_next.side_effect = (True, True, False)
        raw.page_iterator_bounding_box.return_value = (True, (0, 0, 0, 0))
        raw.result_iterator_get_utf8_text.side_effect = ("word1", "word2",
                                                         "word3")
        raw.page_iterator_is_at_beginning_of.side_effect = (True, False, False)
        raw.page_iterator_is_at_final_element.side_effect = (False, False,
                                                             True)

        with self.assertRaises(TesseractError) as te:
            libtesseract.image_to_string(self.image, builder=self.builder)
        self.assertEqual(te.exception.status, "no script")
        self.assertEqual(te.exception.message, "no script detected")

        raw.init.assert_called_once_with(lang=None)
        raw.get_available_languages.assert_called_once_with(self.handle)
        raw.set_page_seg_mode.assert_called_once_with(
            self.handle, self.builder.tesseract_layout)
        raw.set_debug_file.assert_called_once_with(self.handle, os.devnull)
        raw.set_image.assert_called_once_with(self.handle, self.image)
        self.assertFalse(raw.set_is_numeric.called)
        raw.recognize.assert_called_once_with(self.handle)
        raw.get_iterator.assert_called_once_with(self.handle)


class TestLibTesseractDigitsLineBox(BaseTest):

    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.image = Image.new(mode="RGB", size=(1, 1))
        self.builder = builders.DigitLineBoxBuilder()
        self.handle = randint(0, 2**32-1)
        self.iterator = randint(0, 2**32-1)

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_line(self, raw):
        raw.init.return_value = self.handle
        raw.get_iterator.return_value = self.iterator
        raw.result_iterator_get_page_iterator.return_value = self.iterator
        raw.get_available_languages.return_value = ["eng", "fra", "jpn", "osd"]
        raw.page_iterator_next.side_effect = (True, True, False)
        raw.page_iterator_bounding_box.return_value = (True, (0, 0, 0, 0))
        raw.result_iterator_get_utf8_text.side_effect = ("1", "2", "42")
        raw.page_iterator_is_at_beginning_of.side_effect = (True, False, False)
        raw.page_iterator_is_at_final_element.side_effect = (False, False,
                                                             True)

        self.assertListEqual(
            libtesseract.image_to_string(self.image, builder=self.builder),
            [
                builders.LineBox([
                    builders.Box("1", ((0, 0), (0, 0))),
                    builders.Box("2", ((0, 0), (0, 0))),
                    builders.Box("42", ((0, 0), (0, 0))),
                ], ((0, 0), (0, 0)))
            ]
        )

        raw.init.assert_called_once_with(lang=None)
        raw.get_available_languages.assert_called_once_with(self.handle)
        raw.set_page_seg_mode.assert_called_once_with(
            self.handle, self.builder.tesseract_layout)
        raw.set_debug_file.assert_called_once_with(self.handle, os.devnull)
        raw.set_image.assert_called_once_with(self.handle, self.image)
        raw.set_is_numeric.assert_called_once_with(self.handle, True)
        raw.recognize.assert_called_once_with(self.handle)
        raw.get_iterator.assert_called_once_with(self.handle)
        raw.result_iterator_get_page_iterator.assert_called_once_with(
            self.iterator
        )
        self.assertEqual(raw.page_iterator_is_at_beginning_of.call_count, 3)
        raw.page_iterator_is_at_beginning_of.assert_called_with(
            self.iterator, raw.PageIteratorLevel.TEXTLINE)

        # called first at beginning and three times for each word
        self.assertEqual(raw.page_iterator_bounding_box.call_count, 4)
        self.assertListEqual(
            raw.page_iterator_bounding_box.call_args_list,
            [
                call(self.iterator, raw.PageIteratorLevel.TEXTLINE),
                call(self.iterator, raw.PageIteratorLevel.WORD),
                call(self.iterator, raw.PageIteratorLevel.WORD),
                call(self.iterator, raw.PageIteratorLevel.WORD),
            ]
        )
        self.assertEqual(raw.page_iterator_is_at_final_element.call_count, 3)
        raw.page_iterator_is_at_final_element.assert_called_with(
            self.iterator, raw.PageIteratorLevel.TEXTLINE,
            raw.PageIteratorLevel.WORD
        )
        self.assertEqual(raw.result_iterator_get_utf8_text.call_count, 3)
        raw.result_iterator_get_utf8_text.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        self.assertEqual(raw.result_iterator_get_confidence.call_count, 3)
        raw.result_iterator_get_confidence.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        self.assertEqual(raw.page_iterator_next.call_count, 3)
        raw.page_iterator_next.assert_called_with(
            self.iterator, raw.PageIteratorLevel.WORD)
        raw.cleanup.assert_called_once_with(self.handle)

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_line_error(self, raw):
        raw.init.return_value = self.handle
        raw.get_iterator.return_value = None
        raw.result_iterator_get_page_iterator.return_value = self.iterator
        raw.get_available_languages.return_value = ["eng", "fra", "jpn", "osd"]
        raw.page_iterator_next.side_effect = (True, True, False)
        raw.page_iterator_bounding_box.return_value = (True, (0, 0, 0, 0))
        raw.result_iterator_get_utf8_text.side_effect = ("1", "2", "42")
        raw.page_iterator_is_at_beginning_of.side_effect = (True, False, False)
        raw.page_iterator_is_at_final_element.side_effect = (False, False,
                                                             True)

        with self.assertRaises(TesseractError) as te:
            libtesseract.image_to_string(self.image, builder=self.builder)
        self.assertEqual(te.exception.status, "no script")
        self.assertEqual(te.exception.message, "no script detected")

        raw.init.assert_called_once_with(lang=None)
        raw.get_available_languages.assert_called_once_with(self.handle)
        raw.set_page_seg_mode.assert_called_once_with(
            self.handle, self.builder.tesseract_layout)
        raw.set_debug_file.assert_called_once_with(self.handle, os.devnull)
        raw.set_image.assert_called_once_with(self.handle, self.image)
        raw.set_is_numeric.assert_called_once_with(self.handle, True)
        raw.recognize.assert_called_once_with(self.handle)
        raw.get_iterator.assert_called_once_with(self.handle)


class TestLibTesseractPDF(BaseTest):

    def setUp(self):
        self.image = Image.new(mode="RGB", size=(1, 1))
        self.handle = 1234567

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_pdf(self, raw):
        renderer = 2345671
        raw.init.return_value = self.handle
        raw.init_pdf_renderer.return_value = renderer
        libtesseract.image_to_pdf(self.image, "output")

        raw.init.assert_called_once_with(lang=None)
        raw.set_image.assert_called_once_with(self.handle, self.image)
        raw.set_page_seg_mode.assert_called_once_with(
            self.handle, raw.PageSegMode.AUTO_OSD
        )
        raw.recognize.assert_called_once_with(self.handle)
        raw.init_pdf_renderer.assert_called_once_with(
            self.handle, "output", False
        )
        raw.begin_document.assert_called_once_with(renderer, "")
        raw.add_renderer_image.assert_called_once_with(self.handle,
                                                       renderer)
        raw.end_document.assert_called_once_with(renderer)
        self.assertListEqual(
            raw.cleanup.call_args_list,
            [call(self.handle), call(renderer)]
        )

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_multipage_pdf(self, raw):
        renderer = 2345671
        raw.init.return_value = self.handle
        raw.init_pdf_renderer.return_value = renderer
        libtesseract.LibtesseractPdfBuilder() \
            .set_output_file("output")\
            .add_image(self.image)\
            .add_image(self.image)\
            .build()

        raw.init.assert_called_once_with(lang=None)
        raw.set_image.assert_called_with(self.handle, self.image)
        raw.set_image.assert_called_with(self.handle, self.image)
        raw.set_page_seg_mode.assert_called_once_with(
            self.handle, raw.PageSegMode.AUTO_OSD
        )
        raw.recognize.assert_called_with(self.handle)
        raw.recognize.assert_called_with(self.handle)
        raw.init_pdf_renderer.assert_called_once_with(
            self.handle, "output", False
        )
        raw.begin_document.assert_called_once_with(renderer, "")
        raw.add_renderer_image.assert_called_with(
            self.handle, renderer
        )
        raw.add_renderer_image.assert_called_with(
            self.handle, renderer
        )
        raw.end_document.assert_called_once_with(renderer)
        self.assertListEqual(
            raw.cleanup.call_args_list,
            [call(self.handle), call(renderer)]
        )

    @patch("pyocr.libtesseract.tesseract_raw")
    def test_pdf_renderer_error(self, raw):
        renderer = None
        raw.init.return_value = self.handle
        raw.init_pdf_renderer.return_value = renderer

        with self.assertRaises(AssertionError):
            libtesseract.image_to_pdf(self.image, "output")

        raw.init.assert_called_once_with(lang=None)
        raw.set_page_seg_mode.assert_called_once_with(
            self.handle, raw.PageSegMode.AUTO_OSD
        )
        raw.init_pdf_renderer.assert_called_once_with(
            self.handle, "output", False
        )
        self.assertFalse(raw.set_image.called)
        self.assertFalse(raw.set_input_name.called)
        self.assertFalse(raw.recognize.called)
        self.assertFalse(raw.begin_document.called)
        self.assertFalse(raw.add_renderer_image.called)
        self.assertFalse(raw.end_document.called)
        raw.cleanup.assert_called_once_with(self.handle)



#!/usr/bin/env python3

import os
import subprocess
import sys
from setuptools import setup


# NOTE: This file must remain Python 2 compatible for the foreseeable future,
# to ensure that we error out properly for people with outdated setuptools
# and/or pip.
if sys.version_info < (3, 4):
    error = """
Beginning with PyOCR 0.7, Python 3.4 or above is required.

This may be due to an out of date pip.

Make sure you have pip >= 9.0.1.
"""
    sys.exit(error)


if os.name == 'nt':
    setup_deps = []
    scm_version = {}
    # setuptools_scm doesn't work in MSYS2
    if not os.path.exists('src/pyocr/_version.py'):
        version = subprocess.run(
            'git describe --always',
            stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )
        version = version.stdout.decode("utf-8").strip()
        version = version.split("-")[0]

        with open('src/pyocr/_version.py', 'w') as fd:
            fd.write("version = '{}'\n".format(version))
    else:
        with open("src/pyocr/_version.py", "r") as fd:
            for line in fd.readlines():
                if line[0] != '#' and line.strip() != '':
                    version = line.strip()
                    version = version.split(" ")[2][1:-1]
                    break
else:
    setup_deps = [
        'setuptools_scm',
        'setuptools_scm_git_archive',
    ]
    scm_version = {
        'write_to': 'src/pyocr/_version.py',
    }
    version = None


setup(
    name="pyocr",
    description=(
        "A Python wrapper for OCR engines (Tesseract, Cuneiform, etc)"
    ),
    long_description=(
        "A Python wrapper for OCR engines (Tesseract, Cuneiform, etc)"
    ),
    keywords="tesseract cuneiform ocr",
    version=version,
    url="https://gitlab.gnome.org/World/OpenPaperwork/pyocr",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 or later"
        " (GPLv3+)",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Multimedia :: Graphics :: Capture :: Scanners",
        "Topic :: Multimedia :: Graphics :: Graphics Conversion",
        "Topic :: Scientific/Engineering :: Image Recognition",
    ],
    license="GPLv3+",
    author="Jerome Flesch",
    author_email="jflesch@openpaper.work",
    packages=[
        'pyocr',
        'pyocr.libtesseract',
    ],
    package_dir={
        '': 'src',
    },
    data_files=[],
    scripts=[],
    zip_safe=(os.name != 'nt'),
    python_requires='>=3.4',
    install_requires=[
        "Pillow",
    ],
    setup_requires=setup_deps,
    use_scm_version=scm_version,
)



import subprocess

from io import StringIO
from unittest.mock import patch, MagicMock

from PIL import Image

from pyocr import builders
from pyocr import cuneiform

from .tests_base import BaseTest


class TestCuneiform(BaseTest):
    """
    These tests make sure the requirements for the tests are met.
    """

    @patch("shutil.which")
    def test_available(self, which):
        # XXX is it useful?
        which.return_value = True
        self.assertTrue(cuneiform.is_available())
        which.assert_called_once_with("cuneiform")

    @patch("subprocess.Popen")
    def test_version(self, popen):
        stdout = MagicMock()
        stdout.stdout.read.return_value = (
            b"Cuneiform for Linux 1.1.0\n"
            b"Usage: cuneiform [-l languagename -f format --dotmatrix --fax"
            b" --singlecolumn -o result_file] imagefile"
        )
        popen.return_value = stdout
        self.assertSequenceEqual(cuneiform.get_version(), (1, 1, 0))

    @patch("subprocess.Popen")
    def test_version_error(self, popen):
        stdout = MagicMock()
        stdout.stdout.read.return_value = b"\n"
        popen.return_value = stdout
        self.assertIsNone(cuneiform.get_version())

    @patch("subprocess.Popen")
    def test_langs(self, popen):
        stdout = MagicMock()
        stdout.stdout.read.return_value = (
            b"Cuneiform for Linux 1.1.0\n"
            b"Supported languages: eng ger fra rus swe spa ita ruseng ukr srp "
            b"hrv pol dan por dut cze rum hun bul slv lav lit est tur."
        )
        popen.return_value = stdout
        langs = cuneiform.get_available_languages()
        self.assertIn("eng", langs)
        self.assertIn("fra", langs)
        popen.assert_called_once_with(
            ["cuneiform", "-l"],
            stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )

    def test_name(self):
        self.assertEqual(cuneiform.get_name(), "Cuneiform (sh)")

    def test_can_detect_orientation(self):
        self.assertFalse(cuneiform.can_detect_orientation())

    def test_available_builders(self):
        self.assertListEqual(
            cuneiform.get_available_builders(),
            [
                builders.TextBuilder,
                builders.WordBoxBuilder,
                builders.LineBoxBuilder,
            ]
        )


class TestCuneiformTxt(BaseTest):
    """
    These tests make sure the "usual" OCR works fine. (the one generating
    a .txt file)
    """
    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.builder = builders.TextBuilder()
        self.image = Image.new(mode="RGB", size=(1, 1))
        self.text_file = StringIO(self._get_file_content("text"))
        self.stdout = MagicMock()
        self.stdout.stdout.read.return_value = b"Cuneiform for Linux 1.1.0\n"
        self.stdout.wait.return_value = 0
        self.tmp_filename = "/tmp/cuneiform_n0qfk87otxt"
        self.enter = MagicMock()
        self.enter.__enter__.return_value = MagicMock()
        self.enter.__enter__.return_value.configure_mock(
            name=self.tmp_filename
        )

    @patch("pyocr.tesseract.get_version")
    @patch("pyocr.cuneiform.temp_file")
    @patch("codecs.open")
    @patch("subprocess.Popen")
    def test_image_to_string_defaults_to_text_buidler(self, popen, copen,
                                                      temp_file, get_version):
        get_version.return_value = (4, 0, 0)
        popen.return_value = self.stdout
        copen.return_value = self.text_file
        temp_file.return_value = self.enter
        output = cuneiform.image_to_string(self.image)
        self.assertEqual(output, self._get_file_content("text").strip())
        popen.assert_called_once_with(
            ["cuneiform", "-f", "text", "-o", self.tmp_filename, "-"],
            stdin=subprocess.PIPE, stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT
        )

    @patch("pyocr.cuneiform.temp_file")
    @patch("codecs.open")
    @patch("subprocess.Popen")
    def test_lang(self, popen, copen, temp_file):
        popen.return_value = self.stdout
        copen.return_value = self.text_file
        temp_file.return_value = self.enter
        output = cuneiform.image_to_string(self.image, lang="fra",
                                           builder=self.builder)
        self.assertEqual(output, self._get_file_content("text").strip())
        popen.assert_called_once_with(
            ["cuneiform", "-l", "fra", "-f", "text", "-o", self.tmp_filename,
             "-"],
            stdin=subprocess.PIPE, stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT
        )

    @patch("pyocr.cuneiform.temp_file")
    @patch("codecs.open")
    @patch("subprocess.Popen")
    def test_text(self, popen, copen, temp_file):
        popen.return_value = self.stdout
        copen.return_value = self.text_file
        temp_file.return_value = self.enter
        output = cuneiform.image_to_string(self.image,
                                           builder=self.builder)
        self.assertEqual(output, self._get_file_content("text").strip())
        popen.assert_called_once_with(
            ["cuneiform", "-f", "text", "-o", self.tmp_filename, "-"],
            stdin=subprocess.PIPE, stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT
        )

    @patch("subprocess.Popen")
    def test_text_error(self, popen):
        message = ("Cuneiform for Linux 1.1.0\n"
                   "Magick: Improper image header (example.png) reported by "
                   "coders/png.c:2932 (ReadPNGImage)\n")
        self.stdout.stdout.read.return_value = message.encode()
        self.stdout.wait.return_value = 1
        popen.return_value = self.stdout
        with self.assertRaises(cuneiform.CuneiformError) as ce:
            cuneiform.image_to_string(self.image, builder=self.builder)
        self.assertEqual(ce.exception.status, 1)
        self.assertEqual(ce.exception.message, message)

    @patch("pyocr.cuneiform.temp_file")
    @patch("codecs.open")
    @patch("subprocess.Popen")
    def test_text_non_rgb_image(self, popen, copen, temp_file):
        """This tests that image_to_string works with non RGB mode images and
        that image is converted in function."""
        image = self.image.convert("L")
        popen.return_value = self.stdout
        copen.return_value = self.text_file
        temp_file.return_value = self.enter
        output = cuneiform.image_to_string(image, builder=self.builder)
        self.assertEqual(output, self._get_file_content("text").strip())
        popen.assert_called_once_with(
            ["cuneiform", "-f", "text", "-o", self.tmp_filename, "-"],
            stdin=subprocess.PIPE, stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT
        )


class TestCuneiformDigits(BaseTest):

    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.builder = builders.DigitBuilder()
        self.image = Image.new(mode="RGB", size=(1, 1))

    def test_digits_not_implemented(self):
        with self.assertRaises(NotImplementedError):
            cuneiform.image_to_string(self.image, builder=self.builder)

    def test_digits_box_not_implemented(self):
        with self.assertRaises(NotImplementedError):
            cuneiform.image_to_string(self.image,
                                      builder=self.builder)


class TestCuneiformWordBox(BaseTest):
    """
    These tests make sure that cuneiform box handling works fine.
    """
    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.builder = builders.WordBoxBuilder()
        self.image = Image.new(mode="RGB", size=(1, 1))
        self.text_file = StringIO(self._get_file_content("cuneiform.words"))
        self.stdout = MagicMock()
        self.stdout.stdout.read.return_value = b"Cuneiform for Linux 1.1.0\n"
        self.stdout.wait.return_value = 0
        self.tmp_filename = "/tmp/cuneiform_n0qfk87otxt"
        self.enter = MagicMock()
        self.enter.__enter__.return_value = MagicMock()
        self.enter.__enter__.return_value.configure_mock(
            name=self.tmp_filename
        )

    @patch("pyocr.cuneiform.temp_file")
    @patch("codecs.open")
    @patch("subprocess.Popen")
    def test_word(self, popen, copen, temp_file):
        popen.return_value = self.stdout
        copen.return_value = self.text_file
        temp_file.return_value = self.enter
        output = cuneiform.image_to_string(self.image,
                                           builder=self.builder)
        popen.assert_called_once_with(
            ["cuneiform", "-f", "hocr", "-o", self.tmp_filename, "-"],
            stdin=subprocess.PIPE, stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT
        )
        for box in output:
            self.assertIsInstance(box, builders.Box)

    @patch("subprocess.Popen")
    def test_word_error(self, popen):
        stdout = MagicMock()
        message = ("Cuneiform for Linux 1.1.0\n"
                   "Magick: Improper image header (example.png) reported by "
                   "coders/png.c:2932 (ReadPNGImage)\n")
        stdout.stdout.read.return_value = message.encode()
        stdout.wait.return_value = 1
        popen.return_value = stdout
        with self.assertRaises(cuneiform.CuneiformError) as ce:
            cuneiform.image_to_string(self.image,
                                      builder=self.builder)
        self.assertEqual(ce.exception.status, 1)
        self.assertEqual(ce.exception.message, message)


class TestCuneiformLineBox(BaseTest):
    """
    These tests make sure that cuneiform box handling works fine.
    """
    @patch("pyocr.tesseract.get_version")
    def setUp(self, get_version):
        get_version.return_value = (4, 0, 0)
        self.builder = builders.LineBoxBuilder()
        self.image = Image.new(mode="RGB", size=(1, 1))
        self.text_file = StringIO(self._get_file_content("cuneiform.lines"))
        self.stdout = MagicMock()
        self.stdout.stdout.read.return_value = b"Cuneiform for Linux 1.1.0\n"
        self.stdout.wait.return_value = 0
        self.tmp_filename = "/tmp/cuneiform_n0qfk87otxt"
        self.enter = MagicMock()
        self.enter.__enter__.return_value = MagicMock()
        self.enter.__enter__.return_value.configure_mock(
            name=self.tmp_filename
        )

    @patch("pyocr.cuneiform.temp_file")
    @patch("codecs.open")
    @patch("subprocess.Popen")
    def test_line(self, popen, copen, temp_file):
        popen.return_value = self.stdout
        copen.return_value = self.text_file
        temp_file.return_value = self.enter
        output = cuneiform.image_to_string(self.image,
                                           builder=self.builder)
        popen.assert_called_once_with(
            ["cuneiform", "-f", "hocr", "-o", self.tmp_filename, "-"],
            stdin=subprocess.PIPE, stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT
        )
        for box in output:
            self.assertIsInstance(box, builders.LineBox)

    @patch("subprocess.Popen")
    def test_line_error(self, popen):
        message = ("Cuneiform for Linux 1.1.0\n"
                   "Magick: Improper image header (example.png) reported by "
                   "coders/png.c:2932 (ReadPNGImage)\n")
        self.stdout.stdout.read.return_value = message.encode()
        self.stdout.wait.return_value = 1
        popen.return_value = self.stdout
        with self.assertRaises(cuneiform.CuneiformError) as ce:
            cuneiform.image_to_string(self.image,
                                      builder=self.builder)
        self.assertEqual(ce.exception.status, 1)
        self.assertEqual(ce.exception.message, message)
